@@ -23,6 +23,7 @@
 
 #include "gactiongroup.h"
 #include "gactiongroupexporter.h"
+#include "gremoteactiongroup.h"
 #include "gdbusactiongroup.h"
 #include "gapplication.h"
 #include "gfile.h"
@@ -86,6 +87,7 @@ static GDBusInterfaceInfo *org_gtk_private_CommandLine;
 struct _GApplicationImpl
 {
   GDBusConnection *session_bus;
+  GActionGroup    *exported_actions;
   const gchar     *bus_name;
 
   gchar           *object_path;
@@ -301,11 +303,14 @@ g_application_impl_app_menu_changed (GObject    *source,
                                      gpointer    user_data)
 {
   GApplicationImpl *impl = user_data;
+  GMenuModel *app_menu;
 
   g_assert (source == impl->app);
 
-  g_application_impl_publish_menu (impl, "AppMenu", g_application_get_app_menu (impl->app),
-                                   &impl->app_menu_id, &impl->app_menu_path);
+  g_object_get (impl->app, "app-menu", &app_menu, NULL);
+
+  g_application_impl_publish_menu (impl, "AppMenu", app_menu, &impl->app_menu_id, &impl->app_menu_path);
+  g_clear_object (&app_menu);
 }
 
 static void
@@ -314,11 +319,14 @@ g_application_impl_menubar_changed (GObject    *source,
                                     gpointer    user_data)
 {
   GApplicationImpl *impl = user_data;
+  GMenuModel *menubar;
 
   g_assert (source == impl->app);
 
-  g_application_impl_publish_menu (impl, "MenuBar", g_application_get_menubar (impl->app),
-                                   &impl->menubar_id, &impl->menubar_path);
+  g_object_get (impl->app, "menubar", &menubar, NULL);
+
+  g_application_impl_publish_menu (impl, "MenuBar", menubar, &impl->menubar_id, &impl->menubar_path);
+  g_clear_object (&menubar);
 }
 
 /* Attempt to become the primary instance.
@@ -376,7 +384,8 @@ g_application_impl_attempt_primary (GApplicationImpl  *impl,
   if (impl->object_id == 0)
     return FALSE;
 
-  impl->actions_id = g_dbus_connection_export_action_group (impl->session_bus, impl->object_path, impl->app, error);
+  impl->actions_id = g_dbus_connection_export_action_group (impl->session_bus, impl->object_path,
+                                                            impl->exported_actions, error);
 
   if (impl->actions_id == 0)
     return FALSE;
@@ -473,12 +482,13 @@ g_application_impl_destroy (GApplicationImpl *impl)
 }
 
 GApplicationImpl *
-g_application_impl_register (GApplication       *application,
-                             const gchar        *appid,
-                             GApplicationFlags   flags,
-                             GActionGroup      **remote_actions,
-                             GCancellable       *cancellable,
-                             GError            **error)
+g_application_impl_register (GApplication        *application,
+                             const gchar         *appid,
+                             GApplicationFlags    flags,
+                             GActionGroup        *exported_actions,
+                             GRemoteActionGroup **remote_actions,
+                             GCancellable        *cancellable,
+                             GError             **error)
 {
   GDBusActionGroup *actions;
   GApplicationImpl *impl;
@@ -486,6 +496,7 @@ g_application_impl_register (GApplication       *application,
   impl = g_slice_new0 (GApplicationImpl);
 
   impl->app = application;
+  impl->exported_actions = exported_actions;
   impl->bus_name = appid;
 
   impl->session_bus = g_bus_get_sync (G_BUS_TYPE_SESSION, cancellable, NULL);
@@ -542,7 +553,7 @@ g_application_impl_register (GApplication       *application,
       return NULL;
     }
 
-  *remote_actions = G_ACTION_GROUP (actions);
+  *remote_actions = G_REMOTE_ACTION_GROUP (actions);
 
   return impl;
 }
