@@ -2,12 +2,10 @@
  * 
  * Copyright (C) 2006-2007 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,32 +13,42 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Christian Kellner <gicmo@gnome.org> 
  */
 
 #include "config.h"
 #include "gmemoryinputstream.h"
-#include "gpollableinputstream.h"
 #include "ginputstream.h"
 #include "gseekable.h"
 #include "string.h"
-#include "gtask.h"
+#include "gsimpleasyncresult.h"
 #include "gioerror.h"
 #include "glibintl.h"
 
 
 /**
- * GMemoryInputStream:
+ * SECTION:gmemoryinputstream
+ * @short_description: Streaming input operations on memory chunks
+ * @include: gio/gio.h
+ * @see_also: #GMemoryOutputStream
  *
- * `GMemoryInputStream` is a class for using arbitrary
+ * #GMemoryInputStream is a class for using arbitrary
  * memory chunks as input for GIO streaming input operations.
  *
- * As of GLib 2.34, `GMemoryInputStream` implements
- * [iface@Gio.PollableInputStream].
  */
 
+typedef struct _Chunk Chunk;
+
+struct _Chunk {
+  guint8         *data;
+  gsize           len;
+  GDestroyNotify  destroy;
+};
+
 struct _GMemoryInputStreamPrivate {
   GSList *chunks;
   gsize   len;
@@ -59,6 +67,16 @@ static gssize   g_memory_input_stream_skip         (GInputStream         *stream
 static gboolean g_memory_input_stream_close        (GInputStream         *stream,
 						    GCancellable         *cancellable,
 						    GError              **error);
+static void     g_memory_input_stream_read_async   (GInputStream         *stream,
+						    void                 *buffer,
+						    gsize                 count,
+						    int                   io_priority,
+						    GCancellable         *cancellable,
+						    GAsyncReadyCallback   callback,
+						    gpointer              user_data);
+static gssize   g_memory_input_stream_read_finish  (GInputStream         *stream,
+						    GAsyncResult         *result,
+						    GError              **error);
 static void     g_memory_input_stream_skip_async   (GInputStream         *stream,
 						    gsize                 count,
 						    int                   io_priority,
@@ -90,21 +108,11 @@ static gboolean g_memory_input_stream_truncate            (GSeekable       *seek
                                                            goffset          offset,
                                                            GCancellable    *cancellable,
                                                            GError         **error);
-
-static void     g_memory_input_stream_pollable_iface_init (GPollableInputStreamInterface *iface);
-static gboolean g_memory_input_stream_is_readable         (GPollableInputStream *stream);
-static GSource *g_memory_input_stream_create_source       (GPollableInputStream *stream,
-							   GCancellable          *cancellable);
-
 static void     g_memory_input_stream_finalize            (GObject         *object);
 
 G_DEFINE_TYPE_WITH_CODE (GMemoryInputStream, g_memory_input_stream, G_TYPE_INPUT_STREAM,
-                         G_ADD_PRIVATE (GMemoryInputStream)
                          G_IMPLEMENT_INTERFACE (G_TYPE_SEEKABLE,
-                                                g_memory_input_stream_seekable_iface_init);
-                         G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
-                                                g_memory_input_stream_pollable_iface_init);
-			 )
+                                                g_memory_input_stream_seekable_iface_init))
 
 
 static void
@@ -113,6 +121,8 @@ g_memory_input_stream_class_init (GMemoryInputStreamClass *klass)
   GObjectClass *object_class;
   GInputStreamClass *istream_class;
 
+  g_type_class_add_private (klass, sizeof (GMemoryInputStreamPrivate));
+
   object_class = G_OBJECT_CLASS (klass);
   object_class->finalize     = g_memory_input_stream_finalize;
   
@@ -121,12 +131,26 @@ g_memory_input_stream_class_init (GMemoryInputStreamClass *klass)
   istream_class->skip  = g_memory_input_stream_skip;
   istream_class->close_fn = g_memory_input_stream_close;
 
+  istream_class->read_async  = g_memory_input_stream_read_async;
+  istream_class->read_finish  = g_memory_input_stream_read_finish;
   istream_class->skip_async  = g_memory_input_stream_skip_async;
   istream_class->skip_finish  = g_memory_input_stream_skip_finish;
   istream_class->close_async = g_memory_input_stream_close_async;
   istream_class->close_finish = g_memory_input_stream_close_finish;
 }
 
+static void
+free_chunk (gpointer data, 
+            gpointer user_data)
+{
+  Chunk *chunk = data;
+
+  if (chunk->destroy)
+    chunk->destroy (chunk->data);
+
+  g_slice_free (Chunk, chunk);
+}
+
 static void
 g_memory_input_stream_finalize (GObject *object)
 {
@@ -136,7 +160,8 @@ g_memory_input_stream_finalize (GObject *object)
   stream = G_MEMORY_INPUT_STREAM (object);
   priv = stream->priv;
 
-  g_slist_free_full (priv->chunks, (GDestroyNotify)g_bytes_unref);
+  g_slist_foreach (priv->chunks, free_chunk, NULL);
+  g_slist_free (priv->chunks);
 
   G_OBJECT_CLASS (g_memory_input_stream_parent_class)->finalize (object);
 }
@@ -151,17 +176,12 @@ g_memory_input_stream_seekable_iface_init (GSeekableIface *iface)
   iface->truncate_fn  = g_memory_input_stream_truncate;
 }
 
-static void
-g_memory_input_stream_pollable_iface_init (GPollableInputStreamInterface *iface)
-{
-  iface->is_readable   = g_memory_input_stream_is_readable;
-  iface->create_source = g_memory_input_stream_create_source;
-}
-
 static void
 g_memory_input_stream_init (GMemoryInputStream *stream)
 {
-  stream->priv = g_memory_input_stream_get_instance_private (stream);
+  stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
+                                              G_TYPE_MEMORY_INPUT_STREAM,
+                                              GMemoryInputStreamPrivate);
 }
 
 /**
@@ -183,9 +203,9 @@ g_memory_input_stream_new (void)
 
 /**
  * g_memory_input_stream_new_from_data:
- * @data: (array length=len) (element-type guint8) (transfer full): input data
+ * @data: (array length=len) (element-type guint8): input data
  * @len: length of the data, may be -1 if @data is a nul-terminated string
- * @destroy: (nullable): function that is called to free @data, or %NULL
+ * @destroy: (allow-none): function that is called to free @data, or %NULL
  *
  * Creates a new #GMemoryInputStream with data in memory of a given size.
  * 
@@ -206,36 +226,12 @@ g_memory_input_stream_new_from_data (const void     *data,
   return stream;
 }
 
-/**
- * g_memory_input_stream_new_from_bytes:
- * @bytes: a #GBytes
- *
- * Creates a new #GMemoryInputStream with data from the given @bytes.
- *
- * Returns: new #GInputStream read from @bytes
- *
- * Since: 2.34
- **/
-GInputStream *
-g_memory_input_stream_new_from_bytes (GBytes  *bytes)
-{
-  
-  GInputStream *stream;
-
-  stream = g_memory_input_stream_new ();
-
-  g_memory_input_stream_add_bytes (G_MEMORY_INPUT_STREAM (stream),
-				   bytes);
-
-  return stream;
-}
-
 /**
  * g_memory_input_stream_add_data:
  * @stream: a #GMemoryInputStream
- * @data: (array length=len) (element-type guint8) (transfer full): input data
+ * @data: (array length=len) (element-type guint8): input data
  * @len: length of the data, may be -1 if @data is a nul-terminated string
- * @destroy: (nullable): function that is called to free @data, or %NULL
+ * @destroy: (allow-none): function that is called to free @data, or %NULL
  *
  * Appends @data to data that can be read from the input stream
  */
@@ -244,44 +240,25 @@ g_memory_input_stream_add_data (GMemoryInputStream *stream,
                                 const void         *data,
                                 gssize              len,
                                 GDestroyNotify      destroy)
-{
-  GBytes *bytes;
-
-  if (len == -1)
-    len = strlen (data);
-
-  /* It's safe to discard the const here because we're chaining the
-   * destroy callback.
-   */
-  bytes = g_bytes_new_with_free_func (data, len, destroy, (void*)data);
-
-  g_memory_input_stream_add_bytes (stream, bytes);
-  
-  g_bytes_unref (bytes);
-}
-
-/**
- * g_memory_input_stream_add_bytes:
- * @stream: a #GMemoryInputStream
- * @bytes: input data
- *
- * Appends @bytes to data that can be read from the input stream.
- *
- * Since: 2.34
- */
-void
-g_memory_input_stream_add_bytes (GMemoryInputStream *stream,
-				 GBytes             *bytes)
 {
   GMemoryInputStreamPrivate *priv;
+  Chunk *chunk;
  
   g_return_if_fail (G_IS_MEMORY_INPUT_STREAM (stream));
-  g_return_if_fail (bytes != NULL);
+  g_return_if_fail (data != NULL);
 
   priv = stream->priv;
 
-  priv->chunks = g_slist_append (priv->chunks, g_bytes_ref (bytes));
-  priv->len += g_bytes_get_size (bytes);
+  if (len == -1)
+    len = strlen (data);
+  
+  chunk = g_slice_new (Chunk);
+  chunk->data = (guint8 *)data;
+  chunk->len = len;
+  chunk->destroy = destroy;
+
+  priv->chunks = g_slist_append (priv->chunks, chunk);
+  priv->len += chunk->len;
 }
 
 static gssize
@@ -294,8 +271,7 @@ g_memory_input_stream_read (GInputStream  *stream,
   GMemoryInputStream *memory_stream;
   GMemoryInputStreamPrivate *priv;
   GSList *l;
-  GBytes *chunk;
-  gsize len;
+  Chunk *chunk;
   gsize offset, start, rest, size;
 
   memory_stream = G_MEMORY_INPUT_STREAM (stream);
@@ -306,13 +282,12 @@ g_memory_input_stream_read (GInputStream  *stream,
   offset = 0;
   for (l = priv->chunks; l; l = l->next) 
     {
-      chunk = (GBytes *)l->data;
-      len = g_bytes_get_size (chunk);
+      chunk = (Chunk *)l->data;
 
-      if (offset + len > priv->pos)
+      if (offset + chunk->len > priv->pos)
         break;
 
-      offset += len;
+      offset += chunk->len;
     }
   
   start = priv->pos - offset;
@@ -320,14 +295,10 @@ g_memory_input_stream_read (GInputStream  *stream,
 
   for (; l && rest > 0; l = l->next)
     {
-      const guint8* chunk_data;
-      chunk = (GBytes *)l->data;
+      chunk = (Chunk *)l->data;
+      size = MIN (rest, chunk->len - start);
 
-      chunk_data = g_bytes_get_data (chunk, &len);
-
-      size = MIN (rest, len - start);
-
-      memcpy ((guint8 *)buffer + (count - rest), chunk_data + start, size);
+      memcpy ((guint8 *)buffer + (count - rest), chunk->data + start, size);
       rest -= size;
 
       start = 0;
@@ -364,6 +335,51 @@ g_memory_input_stream_close (GInputStream  *stream,
   return TRUE;
 }
 
+static void
+g_memory_input_stream_read_async (GInputStream        *stream,
+                                  void                *buffer,
+                                  gsize                count,
+                                  int                  io_priority,
+                                  GCancellable        *cancellable,
+                                  GAsyncReadyCallback  callback,
+                                  gpointer             user_data)
+{
+  GSimpleAsyncResult *simple;
+  GError *error = NULL;
+  gssize nread;
+
+  nread = G_INPUT_STREAM_GET_CLASS (stream)->read_fn (stream,
+						      buffer,
+						      count,
+						      cancellable,
+						      &error);
+  simple = g_simple_async_result_new (G_OBJECT (stream),
+				      callback,
+				      user_data,
+				      g_memory_input_stream_read_async);
+  if (error)
+    g_simple_async_result_take_error (simple, error);
+  else
+    g_simple_async_result_set_op_res_gssize (simple, nread);
+  g_simple_async_result_complete_in_idle (simple);
+  g_object_unref (simple);
+}
+
+static gssize
+g_memory_input_stream_read_finish (GInputStream  *stream,
+                                   GAsyncResult  *result,
+                                   GError       **error)
+{
+  GSimpleAsyncResult *simple;
+  gssize nread;
+
+  simple = G_SIMPLE_ASYNC_RESULT (result);
+  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == g_memory_input_stream_read_async);
+  
+  nread = g_simple_async_result_get_op_res_gssize (simple);
+  return nread;
+}
+
 static void
 g_memory_input_stream_skip_async (GInputStream        *stream,
                                   gsize                count,
@@ -372,19 +388,17 @@ g_memory_input_stream_skip_async (GInputStream        *stream,
                                   GAsyncReadyCallback  callback,
                                   gpointer             user_data)
 {
-  GTask *task;
+  GSimpleAsyncResult *simple;
   gssize nskipped;
-  GError *error = NULL;
 
-  nskipped = G_INPUT_STREAM_GET_CLASS (stream)->skip (stream, count, cancellable, &error);
-  task = g_task_new (stream, cancellable, callback, user_data);
-  g_task_set_source_tag (task, g_memory_input_stream_skip_async);
-
-  if (error)
-    g_task_return_error (task, error);
-  else
-    g_task_return_int (task, nskipped);
-  g_object_unref (task);
+  nskipped = g_input_stream_skip (stream, count, cancellable, NULL);
+  simple = g_simple_async_result_new (G_OBJECT (stream),
+                                      callback,
+                                      user_data,
+                                      g_memory_input_stream_skip_async);
+  g_simple_async_result_set_op_res_gssize (simple, nskipped);
+  g_simple_async_result_complete_in_idle (simple);
+  g_object_unref (simple);
 }
 
 static gssize
@@ -392,9 +406,14 @@ g_memory_input_stream_skip_finish (GInputStream  *stream,
                                    GAsyncResult  *result,
                                    GError       **error)
 {
-  g_return_val_if_fail (g_task_is_valid (result, stream), -1);
+  GSimpleAsyncResult *simple;
+  gssize nskipped;
 
-  return g_task_propagate_int (G_TASK (result), error);
+  simple = G_SIMPLE_ASYNC_RESULT (result);
+  g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == g_memory_input_stream_skip_async);
+  
+  nskipped = g_simple_async_result_get_op_res_gssize (simple);
+  return nskipped;
 }
 
 static void
@@ -404,12 +423,14 @@ g_memory_input_stream_close_async (GInputStream        *stream,
                                    GAsyncReadyCallback  callback,
                                    gpointer             user_data)
 {
-  GTask *task;
-
-  task = g_task_new (stream, cancellable, callback, user_data);
-  g_task_set_source_tag (task, g_memory_input_stream_close_async);
-  g_task_return_boolean (task, TRUE);
-  g_object_unref (task);
+  GSimpleAsyncResult *simple;
+  
+  simple = g_simple_async_result_new (G_OBJECT (stream),
+				      callback,
+				      user_data,
+				      g_memory_input_stream_close_async);
+  g_simple_async_result_complete_in_idle (simple);
+  g_object_unref (simple);
 }
 
 static gboolean
@@ -475,7 +496,7 @@ g_memory_input_stream_seek (GSeekable     *seekable,
       return FALSE;
     }
 
-  if (absolute < 0 || (gsize) absolute > priv->len)
+  if (absolute < 0 || absolute > priv->len)
     {
       g_set_error_literal (error,
                            G_IO_ERROR,
@@ -507,23 +528,3 @@ g_memory_input_stream_truncate (GSeekable     *seekable,
                        _("Cannot truncate GMemoryInputStream"));
   return FALSE;
 }
-
-static gboolean
-g_memory_input_stream_is_readable (GPollableInputStream *stream)
-{
-  return TRUE;
-}
-
-static GSource *
-g_memory_input_stream_create_source (GPollableInputStream *stream,
-				     GCancellable         *cancellable)
-{
-  GSource *base_source, *pollable_source;
-
-  base_source = g_timeout_source_new (0);
-  pollable_source = g_pollable_source_new_full (stream, base_source,
-						cancellable);
-  g_source_unref (base_source);
-
-  return pollable_source;
-}
