@@ -54,7 +54,6 @@
 #include "gioerror.h"
 #include "gioenums.h"
 #include "gioerror.h"
-#include "gio-marshal.h"
 #include "gnetworkingprivate.h"
 #include "gsocketaddress.h"
 #include "gsocketcontrolmessage.h"
@@ -218,13 +217,15 @@ socket_strerror (int err)
 #ifndef G_OS_WIN32
   return g_strerror (err);
 #else
-  static GStaticPrivate last_msg = G_STATIC_PRIVATE_INIT;
+  const char *msg_ret;
   char *msg;
 
   msg = g_win32_error_message (err);
-  g_static_private_set (&last_msg, msg, g_free);
 
-  return msg;
+  msg_ret = g_intern_string (msg);
+  g_free (msg);
+
+  return msg_ret;
 #endif
 }
 
@@ -310,7 +311,7 @@ g_socket_details_from_fd (GSocket *socket)
   gint fd;
   guint addrlen;
   guint optlen;
-  int value;
+  int value, family;
   int errsv;
 #ifdef G_OS_WIN32
   /* See bug #611756 */
@@ -371,9 +372,32 @@ g_socket_details_from_fd (GSocket *socket)
       goto err;
     }
 
-  g_assert (G_STRUCT_OFFSET (struct sockaddr, sa_family) +
-	    sizeof address.ss_family <= addrlen);
-  switch (address.ss_family)
+  if (addrlen > 0)
+    {
+      g_assert (G_STRUCT_OFFSET (struct sockaddr, sa_family) +
+		sizeof address.ss_family <= addrlen);
+      family = address.ss_family;
+    }
+  else
+    {
+      /* On Solaris, this happens if the socket is not yet connected.
+       * But we can use SO_DOMAIN as a workaround there.
+       */
+#ifdef SO_DOMAIN
+      optlen = sizeof family;
+      if (getsockopt (fd, SOL_SOCKET, SO_DOMAIN, (void *)&family, &optlen) != 0)
+	{
+	  errsv = get_socket_errno ();
+	  goto err;
+	}
+#else
+      /* This will translate to G_IO_ERROR_FAILED on either unix or windows */
+      errsv = -1;
+      goto err;
+#endif
+    }
+
+  switch (family)
     {
      case G_SOCKET_FAMILY_IPV4:
      case G_SOCKET_FAMILY_IPV6:
@@ -1608,7 +1632,7 @@ g_socket_accept (GSocket       *socket,
  * If the connect call needs to do network I/O it will block, unless
  * non-blocking I/O is enabled. Then %G_IO_ERROR_PENDING is returned
  * and the user can be notified of the connection finishing by waiting
- * for the G_IO_OUT condition. The result of the connection can then be
+ * for the G_IO_OUT condition. The result of the connection must then be
  * checked with g_socket_check_connect_result().
  *
  * Returns: %TRUE if connected, %FALSE on error.
@@ -1658,7 +1682,6 @@ g_socket_connect (GSocket         *socket,
 		      if (g_socket_check_connect_result (socket, error))
 			break;
 		    }
-		  g_prefix_error (error, _("Error connecting: "));
 		}
 	      else
                 {
@@ -1668,9 +1691,9 @@ g_socket_connect (GSocket         *socket,
                 }
 	    }
 	  else
-	    g_set_error (error, G_IO_ERROR,
-			 socket_io_error_from_errno (errsv),
-			 _("Error connecting: %s"), socket_strerror (errsv));
+	    g_set_error_literal (error, G_IO_ERROR,
+				 socket_io_error_from_errno (errsv),
+				 socket_strerror (errsv));
 
 	  return FALSE;
 	}
@@ -1728,6 +1751,8 @@ g_socket_check_connect_result (GSocket  *socket,
         }
       return FALSE;
     }
+
+  socket->priv->connected = TRUE;
   return TRUE;
 }
 
@@ -2083,7 +2108,7 @@ g_socket_send_to (GSocket         *socket,
  *
  * Shut down part of a full-duplex connection.
  *
- * If @shutdown_read is %TRUE then the recieving side of the connection
+ * If @shutdown_read is %TRUE then the receiving side of the connection
  * is shut down, and further reading is disallowed.
  *
  * If @shutdown_write is %TRUE then the sending side of the connection
@@ -2438,7 +2463,6 @@ socket_source_prepare (GSource *source,
       if (*timeout < 0)
         {
           socket_source->socket->priv->timed_out = TRUE;
-          socket_source->pollfd.revents = socket_source->condition & (G_IO_IN | G_IO_OUT);
           *timeout = 0;
           return TRUE;
         }
@@ -2475,6 +2499,8 @@ socket_source_dispatch (GSource     *source,
 #ifdef G_OS_WIN32
   socket_source->pollfd.revents = update_condition (socket_source->socket);
 #endif
+  if (socket_source->socket->priv->timed_out)
+    socket_source->pollfd.revents |= socket_source->condition & (G_IO_IN | G_IO_OUT);
 
   return (*func) (socket_source->socket,
 		  socket_source->pollfd.revents & socket_source->condition,
@@ -2509,8 +2535,8 @@ socket_source_closure_callback (GSocket      *socket,
 {
   GClosure *closure = data;
 
-  GValue params[2] = { { 0, }, { 0, } };
-  GValue result_value = { 0, };
+  GValue params[2] = { G_VALUE_INIT, G_VALUE_INIT };
+  GValue result_value = G_VALUE_INIT;
   gboolean result;
 
   g_value_init (&result_value, G_TYPE_BOOLEAN);
@@ -2537,7 +2563,7 @@ static GSourceFuncs socket_source_funcs =
   socket_source_dispatch,
   socket_source_finalize,
   (GSourceFunc)socket_source_closure_callback,
-  (GSourceDummyMarshal)_gio_marshal_BOOLEAN__FLAGS,
+  (GSourceDummyMarshal)g_cclosure_marshal_generic,
 };
 
 static GSource *
