@@ -2,12 +2,10 @@
  * 
  * Copyright (C) 2008 Red Hat, Inc.
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +13,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
@@ -26,7 +26,6 @@
 /**
  * SECTION:gsocketconnectable
  * @short_description: Interface for potential socket endpoints
- * @include: gio/gio.h
  *
  * Objects that describe one or more potential socket endpoints
  * implement #GSocketConnectable. Callers can then use
@@ -34,7 +33,7 @@
  * to try out each socket address in turn until one succeeds, as shown
  * in the sample code below.
  *
- * |[<!-- language="C" -->
+ * |[
  * MyConnectionType *
  * connect_to_host (const char    *hostname,
  *                  guint16        port,
@@ -47,13 +46,14 @@
  *   GSocketAddress *sockaddr;
  *   GError *conn_error = NULL;
  *
- *   addr = g_network_address_new (hostname, port);
+ *   addr = g_network_address_new ("www.gnome.org", 80);
  *   enumerator = g_socket_connectable_enumerate (addr);
  *   g_object_unref (addr);
  *
- *   // Try each sockaddr until we succeed. Record the first connection error,
- *   // but not any further ones (since they'll probably be basically the same
- *   // as the first).
+ *   /<!-- -->* Try each sockaddr until we succeed. Record the first
+ *    * connection error, but not any further ones (since they'll probably
+ *    * be basically the same as the first).
+ *    *<!-- -->/
  *   while (!conn && (sockaddr = g_socket_address_enumerator_next (enumerator, cancellable, error))
  *     {
  *       conn = connect_to_sockaddr (sockaddr, conn_error ? NULL : &conn_error);
@@ -65,15 +65,18 @@
  *     {
  *       if (conn_error)
  *         {
- *           // We couldn't connect to the first address, but we succeeded
- *           // in connecting to a later address.
+ *           /<!-- -->* We couldn't connect to the first address, but we succeeded
+ *            * in connecting to a later address.
+ *            *<!-- -->/
  *           g_error_free (conn_error);
  *         }
  *       return conn;
  *     }
  *   else if (error)
  *     {
- *       /// Either initial lookup failed, or else the caller cancelled us.
+ *       /<!-- -->* Either the initial lookup failed, or else the caller
+ *        * cancelled us.
+ *        *<!-- -->/
  *       if (conn_error)
  *         g_error_free (conn_error);
  *       return NULL;
@@ -102,7 +105,7 @@ g_socket_connectable_default_init (GSocketConnectableInterface *iface)
  *
  * Creates a #GSocketAddressEnumerator for @connectable.
  *
- * Returns: (transfer full): a new #GSocketAddressEnumerator.
+ * Return value: (transfer full): a new #GSocketAddressEnumerator.
  *
  * Since: 2.22
  */
@@ -123,14 +126,14 @@ g_socket_connectable_enumerate (GSocketConnectable *connectable)
  * @connectable: a #GSocketConnectable
  *
  * Creates a #GSocketAddressEnumerator for @connectable that will
- * return a #GProxyAddress for each of its addresses that you must connect
+ * return #GProxyAddress<!-- -->es for addresses that you must connect
  * to via a proxy.
  *
  * If @connectable does not implement
  * g_socket_connectable_proxy_enumerate(), this will fall back to
  * calling g_socket_connectable_enumerate().
  *
- * Returns: (transfer full): a new #GSocketAddressEnumerator.
+ * Return value: (transfer full): a new #GSocketAddressEnumerator.
  *
  * Since: 2.26
  */
@@ -148,34 +151,3 @@ g_socket_connectable_proxy_enumerate (GSocketConnectable *connectable)
   else
     return (* iface->enumerate) (connectable);
 }
-
-/**
- * g_socket_connectable_to_string:
- * @connectable: a #GSocketConnectable
- *
- * Format a #GSocketConnectable as a string. This is a human-readable format for
- * use in debugging output, and is not a stable serialization format. It is not
- * suitable for use in user interfaces as it exposes too much information for a
- * user.
- *
- * If the #GSocketConnectable implementation does not support string formatting,
- * the implementation’s type name will be returned as a fallback.
- *
- * Returns: (transfer full): the formatted string
- *
- * Since: 2.48
- */
-gchar *
-g_socket_connectable_to_string (GSocketConnectable *connectable)
-{
-  GSocketConnectableIface *iface;
-
-  g_return_val_if_fail (G_IS_SOCKET_CONNECTABLE (connectable), NULL);
-
-  iface = G_SOCKET_CONNECTABLE_GET_IFACE (connectable);
-
-  if (iface->to_string != NULL)
-    return iface->to_string (connectable);
-  else
-    return g_strdup (G_OBJECT_TYPE_NAME (connectable));
-}
