@@ -3,10 +3,10 @@
  * Copyright © 2009 Codethink Limited
  * Copyright © 2009 Red Hat, Inc
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published
+ * by the Free Software Foundation; either version 2 of the licence or (at
+ * your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Ryan Lortie <desrt@desrt.ca>
  *          Alexander Larsson <alexl@redhat.com>
@@ -24,7 +26,6 @@
  * SECTION:gsocketservice
  * @title: GSocketService
  * @short_description: Make it easy to implement a network service
- * @include: gio/gio.h
  * @see_also: #GThreadedSocketService, #GSocketListener.
  *
  * A #GSocketService is an object that represents a service that
@@ -47,9 +48,9 @@
  * If you are interested in writing connection handlers that contain
  * blocking code then see #GThreadedSocketService.
  *
- * The socket service runs on the main loop of the 
- * [thread-default context][g-main-context-push-thread-default-context]
- * of the thread it is created in, and is not
+ * The socket service runs on the main loop of the <link
+ * linkend="g-main-context-push-thread-default-context">thread-default
+ * context</link> of the thread it is created in, and is not
  * threadsafe in general. However, the calls to start and stop the
  * service are thread-safe so these can be used from threads that
  * handle incoming clients.
@@ -63,8 +64,13 @@
 #include <gio/gio.h>
 #include "gsocketlistener.h"
 #include "gsocketconnection.h"
-#include "glibintl.h"
-#include "gmarshal-internal.h"
+
+
+static guint g_socket_service_incoming_signal;
+
+G_DEFINE_TYPE (GSocketService, g_socket_service, G_TYPE_SOCKET_LISTENER);
+
+G_LOCK_DEFINE_STATIC(active);
 
 struct _GSocketServicePrivate
 {
@@ -73,18 +79,6 @@ struct _GSocketServicePrivate
   guint outstanding_accept : 1;
 };
 
-static guint g_socket_service_incoming_signal;
-
-G_LOCK_DEFINE_STATIC(active);
-
-G_DEFINE_TYPE_WITH_PRIVATE (GSocketService, g_socket_service, G_TYPE_SOCKET_LISTENER)
-
-enum
-{
-  PROP_0,
-  PROP_ACTIVE
-};
-
 static void g_socket_service_ready (GObject      *object,
 				    GAsyncResult *result,
 				    gpointer      user_data);
@@ -100,7 +94,9 @@ g_socket_service_real_incoming (GSocketService    *service,
 static void
 g_socket_service_init (GSocketService *service)
 {
-  service->priv = g_socket_service_get_instance_private (service);
+  service->priv = G_TYPE_INSTANCE_GET_PRIVATE (service,
+					       G_TYPE_SOCKET_SERVICE,
+					       GSocketServicePrivate);
   service->priv->cancellable = g_cancellable_new ();
   service->priv->active = TRUE;
 }
@@ -125,90 +121,6 @@ do_accept (GSocketService  *service)
   service->priv->outstanding_accept = TRUE;
 }
 
-static gboolean
-get_active (GSocketService *service)
-{
-  gboolean active;
-
-  G_LOCK (active);
-  active = service->priv->active;
-  G_UNLOCK (active);
-
-  return active;
-}
-
-static void
-set_active (GSocketService *service, gboolean active)
-{
-  gboolean notify = FALSE;
-
-  active = !!active;
-
-  G_LOCK (active);
-
-  if (active != service->priv->active)
-    {
-      service->priv->active = active;
-      notify = TRUE;
-
-      if (active)
-        {
-          if (service->priv->outstanding_accept)
-            g_cancellable_cancel (service->priv->cancellable);
-          else
-            do_accept (service);
-        }
-      else
-        {
-          if (service->priv->outstanding_accept)
-            g_cancellable_cancel (service->priv->cancellable);
-        }
-    }
-
-  G_UNLOCK (active);
-
-  if (notify)
-    g_object_notify (G_OBJECT (service), "active");
-}
-
-static void
-g_socket_service_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-  GSocketService *service = G_SOCKET_SERVICE (object);
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE:
-      g_value_set_boolean (value, get_active (service));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-g_socket_service_set_property (GObject      *object,
-                               guint         prop_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-  GSocketService *service = G_SOCKET_SERVICE (object);
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE:
-      set_active (service, g_value_get_boolean (value));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 g_socket_service_changed (GSocketListener *listener)
 {
@@ -221,7 +133,11 @@ g_socket_service_changed (GSocketListener *listener)
       if (service->priv->outstanding_accept)
 	g_cancellable_cancel (service->priv->cancellable);
       else
-	do_accept (service);
+	{
+	  g_socket_listener_accept_async (listener, service->priv->cancellable,
+					  g_socket_service_ready, NULL);
+	  service->priv->outstanding_accept = TRUE;
+	}
     }
 
   G_UNLOCK (active);
@@ -243,19 +159,20 @@ g_socket_service_changed (GSocketListener *listener)
 gboolean
 g_socket_service_is_active (GSocketService *service)
 {
-  g_return_val_if_fail (G_IS_SOCKET_SERVICE (service), FALSE);
+  gboolean active;
 
-  return get_active (service);
+  G_LOCK (active);
+  active = service->priv->active;
+  G_UNLOCK (active);
+  return active;
 }
 
 /**
  * g_socket_service_start:
  * @service: a #GSocketService
  *
- * Restarts the service, i.e. start accepting connections
- * from the added sockets when the mainloop runs. This only needs
- * to be called after the service has been stopped from
- * g_socket_service_stop().
+ * Starts the service, i.e. start accepting connections
+ * from the added sockets when the mainloop runs.
  *
  * This call is thread-safe, so it may be called from a thread
  * handling an incoming client request.
@@ -265,9 +182,19 @@ g_socket_service_is_active (GSocketService *service)
 void
 g_socket_service_start (GSocketService *service)
 {
-  g_return_if_fail (G_IS_SOCKET_SERVICE (service));
+  G_LOCK (active);
 
-  set_active (service, TRUE);
+  if (!service->priv->active)
+    {
+      service->priv->active = TRUE;
+
+      if (service->priv->outstanding_accept)
+	g_cancellable_cancel (service->priv->cancellable);
+      else
+	do_accept (service);
+    }
+
+  G_UNLOCK (active);
 }
 
 /**
@@ -280,26 +207,25 @@ g_socket_service_start (GSocketService *service)
  * This call is thread-safe, so it may be called from a thread
  * handling an incoming client request.
  *
- * Note that this only stops accepting new connections; it does not
- * close the listening sockets, and you can call
- * g_socket_service_start() again later to begin listening again. To
- * close the listening sockets, call g_socket_listener_close(). (This
- * will happen automatically when the #GSocketService is finalized.)
- *
- * This must be called before calling g_socket_listener_close() as
- * the socket service will start accepting connections immediately
- * when a new socket is added.
- *
  * Since: 2.22
  */
 void
 g_socket_service_stop (GSocketService *service)
 {
-  g_return_if_fail (G_IS_SOCKET_SERVICE (service));
+  G_LOCK (active);
 
-  set_active (service, FALSE);
+  if (service->priv->active)
+    {
+      service->priv->active = FALSE;
+
+      if (service->priv->outstanding_accept)
+	g_cancellable_cancel (service->priv->cancellable);
+    }
+
+  G_UNLOCK (active);
 }
 
+
 static gboolean
 g_socket_service_incoming (GSocketService    *service,
                            GSocketConnection *connection,
@@ -318,9 +244,9 @@ g_socket_service_class_init (GSocketServiceClass *class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   GSocketListenerClass *listener_class = G_SOCKET_LISTENER_CLASS (class);
 
+  g_type_class_add_private (class, sizeof (GSocketServicePrivate));
+
   gobject_class->finalize = g_socket_service_finalize;
-  gobject_class->set_property = g_socket_service_set_property;
-  gobject_class->get_property = g_socket_service_get_property;
   listener_class->changed = g_socket_service_changed;
   class->incoming = g_socket_service_real_incoming;
 
@@ -328,8 +254,7 @@ g_socket_service_class_init (GSocketServiceClass *class)
    * GSocketService::incoming:
    * @service: the #GSocketService
    * @connection: a new #GSocketConnection object
-   * @source_object: (nullable): the source_object passed to
-   *     g_socket_listener_add_address()
+   * @source_object: the source_object passed to g_socket_listener_add_address()
    *
    * The ::incoming signal is emitted when a new incoming connection
    * to @service needs to be handled. The handler must initiate the
@@ -344,29 +269,11 @@ g_socket_service_class_init (GSocketServiceClass *class)
    * Since: 2.22
    */
   g_socket_service_incoming_signal =
-    g_signal_new (I_("incoming"), G_TYPE_FROM_CLASS (class), G_SIGNAL_RUN_LAST,
+    g_signal_new ("incoming", G_TYPE_FROM_CLASS (class), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GSocketServiceClass, incoming),
                   g_signal_accumulator_true_handled, NULL,
-                  _g_cclosure_marshal_BOOLEAN__OBJECT_OBJECT,
-                  G_TYPE_BOOLEAN,
+                  NULL, G_TYPE_BOOLEAN,
                   2, G_TYPE_SOCKET_CONNECTION, G_TYPE_OBJECT);
-  g_signal_set_va_marshaller (g_socket_service_incoming_signal,
-                              G_TYPE_FROM_CLASS (class),
-                              _g_cclosure_marshal_BOOLEAN__OBJECT_OBJECTv);
-
-  /**
-   * GSocketService:active:
-   *
-   * Whether the service is currently accepting connections.
-   *
-   * Since: 2.46
-   */
-  g_object_class_install_property (gobject_class, PROP_ACTIVE,
-                                   g_param_spec_boolean ("active",
-                                                         P_("Active"),
-                                                         P_("Whether the service is currently accepting connections"),
-                                                         TRUE,
-                                                         G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -405,6 +312,7 @@ g_socket_service_ready (GObject      *object,
   G_UNLOCK (active);
 }
 
+
 /**
  * g_socket_service_new:
  *
@@ -412,10 +320,6 @@ g_socket_service_ready (GObject      *object,
  * New listeners can be added with e.g. g_socket_listener_add_address()
  * or g_socket_listener_add_inet_port().
  *
- * New services are created active, there is no need to call
- * g_socket_service_start(), unless g_socket_service_stop() has been
- * called before.
- *
  * Returns: a new #GSocketService.
  *
  * Since: 2.22
