@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the licence, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
@@ -23,11 +25,12 @@
 #include <string.h>
 
 struct _GvdbTable {
-  GBytes *bytes;
+  gint ref_count;
 
   const gchar *data;
   gsize size;
 
+  GMappedFile *mapped;
   gboolean byteswapped;
   gboolean trusted;
 
@@ -120,83 +123,25 @@ gvdb_table_setup_root (GvdbTable                 *file,
   file->n_hash_items = size / sizeof (struct gvdb_hash_item);
 }
 
-/**
- * gvdb_table_new_from_bytes:
- * @bytes: the #GBytes with the data
- * @trusted: if the contents of @bytes are trusted
- * @error: %NULL, or a pointer to a %NULL #GError
- *
- * Creates a new #GvdbTable from the contents of @bytes.
- *
- * This call can fail if the header contained in @bytes is invalid or if @bytes
- * is empty; if so, %G_FILE_ERROR_INVAL will be returned.
- *
- * You should call gvdb_table_free() on the return result when you no
- * longer require it.
- *
- * Returns: a new #GvdbTable
- **/
-GvdbTable *
-gvdb_table_new_from_bytes (GBytes    *bytes,
-                           gboolean   trusted,
-                           GError   **error)
-{
-  const struct gvdb_header *header;
-  GvdbTable *file;
-
-  file = g_slice_new0 (GvdbTable);
-  file->bytes = g_bytes_ref (bytes);
-  file->data = g_bytes_get_data (bytes, &file->size);
-  file->trusted = trusted;
-
-  if (file->size < sizeof (struct gvdb_header))
-    goto invalid;
-
-  header = (gpointer) file->data;
-
-  if (header->signature[0] == GVDB_SIGNATURE0 &&
-      header->signature[1] == GVDB_SIGNATURE1 &&
-      guint32_from_le (header->version) == 0)
-    file->byteswapped = FALSE;
-
-  else if (header->signature[0] == GVDB_SWAPPED_SIGNATURE0 &&
-           header->signature[1] == GVDB_SWAPPED_SIGNATURE1 &&
-           guint32_from_le (header->version) == 0)
-    file->byteswapped = TRUE;
-
-  else
-    goto invalid;
-
-  gvdb_table_setup_root (file, &header->root);
-
-  return file;
-
-invalid:
-  g_set_error_literal (error, G_FILE_ERROR, G_FILE_ERROR_INVAL, "invalid gvdb header");
-
-  g_bytes_unref (file->bytes);
-
-  g_slice_free (GvdbTable, file);
-
-  return NULL;
-}
-
 /**
  * gvdb_table_new:
- * @filename: a filename
- * @trusted: if the contents of @bytes are trusted
+ * @filename: the path to the hash file
+ * @trusted: if the contents of @filename are trusted
  * @error: %NULL, or a pointer to a %NULL #GError
+ * @returns: a new #GvdbTable
  *
- * Creates a new #GvdbTable using the #GMappedFile for @filename as the
- * #GBytes.
+ * Creates a new #GvdbTable from the contents of the file found at
+ * @filename.
  *
- * This function will fail if the file cannot be opened.
+ * The only time this function fails is if the file cannot be opened.
  * In that case, the #GError that is returned will be an error from
  * g_mapped_file_new().
  *
- * An empty or corrupt file will result in %G_FILE_ERROR_INVAL.
+ * An empty or otherwise corrupted file is considered to be a valid
+ * #GvdbTable with no entries.
  *
- * Returns: a new #GvdbTable
+ * You should call gvdb_table_unref() on the return result when you no
+ * longer require it.
  **/
 GvdbTable *
 gvdb_table_new (const gchar  *filename,
@@ -204,21 +149,46 @@ gvdb_table_new (const gchar  *filename,
                 GError      **error)
 {
   GMappedFile *mapped;
-  GvdbTable *table;
-  GBytes *bytes;
+  GvdbTable *file;
 
-  mapped = g_mapped_file_new (filename, FALSE, error);
-  if (!mapped)
+  if ((mapped = g_mapped_file_new (filename, FALSE, error)) == NULL)
     return NULL;
 
-  bytes = g_mapped_file_get_bytes (mapped);
-  table = gvdb_table_new_from_bytes (bytes, trusted, error);
-  g_mapped_file_unref (mapped);
-  g_bytes_unref (bytes);
+  file = g_slice_new0 (GvdbTable);
+  file->data = g_mapped_file_get_contents (mapped);
+  file->size = g_mapped_file_get_length (mapped);
+  file->trusted = trusted;
+  file->mapped = mapped;
+  file->ref_count = 1;
 
-  g_prefix_error (error, "%s: ", filename);
+  if (sizeof (struct gvdb_header) <= file->size)
+    {
+      const struct gvdb_header *header = (gpointer) file->data;
 
-  return table;
+      if (header->signature[0] == GVDB_SIGNATURE0 &&
+          header->signature[1] == GVDB_SIGNATURE1 &&
+          guint32_from_le (header->version) == 0)
+        file->byteswapped = FALSE;
+
+      else if (header->signature[0] == GVDB_SWAPPED_SIGNATURE0 &&
+               header->signature[1] == GVDB_SWAPPED_SIGNATURE1 &&
+               guint32_from_le (header->version) == 0)
+        file->byteswapped = TRUE;
+
+      else
+        {
+          g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_INVAL,
+                       "%s: invalid header", filename);
+          g_slice_free (GvdbTable, file);
+          g_mapped_file_unref (mapped);
+
+          return NULL;
+        }
+
+      gvdb_table_setup_root (file, &header->root);
+    }
+
+  return file;
 }
 
 static gboolean
@@ -311,6 +281,18 @@ gvdb_table_lookup (GvdbTable   *file,
   return NULL;
 }
 
+static const struct gvdb_hash_item *
+gvdb_table_get_item (GvdbTable  *table,
+                     guint32_le  item_no)
+{
+  guint32 item_no_native = guint32_from_le (item_no);
+
+  if G_LIKELY (item_no_native < table->n_hash_items)
+    return table->hash_items + item_no_native;
+
+  return NULL;
+}
+
 static gboolean
 gvdb_table_list_from_item (GvdbTable                    *table,
                            const struct gvdb_hash_item  *item,
@@ -329,160 +311,12 @@ gvdb_table_list_from_item (GvdbTable                    *table,
   return TRUE;
 }
 
-/**
- * gvdb_table_get_names:
- * @table: a #GvdbTable
- * @length: the number of items returned, or %NULL
- *
- * Gets a list of all names contained in @table.
- *
- * No call to gvdb_table_get_table(), gvdb_table_list() or
- * gvdb_table_get_value() will succeed unless it is for one of the
- * names returned by this function.
- *
- * Note that some names that are returned may still fail for all of the
- * above calls in the case of the corrupted file.  Note also that the
- * returned strings may not be utf8.
- *
- * Returns: a %NULL-terminated list of strings, of length @length
- **/
-gchar **
-gvdb_table_get_names (GvdbTable *table,
-                      gint      *length)
-{
-  gchar **names;
-  gint n_names;
-  gint filled;
-  gint total;
-  gint i;
-
-  /* We generally proceed by iterating over the list of items in the
-   * hash table (in order of appearance) recording them into an array.
-   *
-   * Each item has a parent item (except root items).  The parent item
-   * forms part of the name of the item.  We could go fetching the
-   * parent item chain at the point that we encounter each item but then
-   * we would need to implement some sort of recursion along with checks
-   * for self-referential items.
-   *
-   * Instead, we do a number of passes.  Each pass will build up one
-   * level of names (starting from the root).  We continue to do passes
-   * until no more items are left.  The first pass will only add root
-   * items and each further pass will only add items whose direct parent
-   * is an item added in the immediately previous pass.  It's also
-   * possible that items get filled if they follow their parent within a
-   * particular pass.
-   *
-   * At most we will have a number of passes equal to the depth of the
-   * tree.  Self-referential items will never be filled in (since their
-   * parent will have never been filled in).  We continue until we have
-   * a pass that fills in no additional items.
-   *
-   * This takes an O(n) algorithm and turns it into O(n*m) where m is
-   * the depth of the tree, but in all sane cases the tree won't be very
-   * deep and the constant factor of this algorithm is lower (and the
-   * complexity of coding it, as well).
-   */
-
-  n_names = table->n_hash_items;
-  names = g_new0 (gchar *, n_names + 1);
-
-  /* 'names' starts out all-NULL.  On each pass we record the number
-   * of items changed from NULL to non-NULL in 'filled' so we know if we
-   * should repeat the loop.  'total' counts the total number of items
-   * filled.  If 'total' ends up equal to 'n_names' then we know that
-   * 'names' has been completely filled.
-   */
-
-  total = 0;
-  do
-    {
-      /* Loop until we have filled no more entries */
-      filled = 0;
-
-      for (i = 0; i < n_names; i++)
-        {
-          const struct gvdb_hash_item *item = &table->hash_items[i];
-          const gchar *name;
-          gsize name_length;
-          guint32 parent;
-
-          /* already got it on a previous pass */
-          if (names[i] != NULL)
-            continue;
-
-          parent = guint32_from_le (item->parent);
-
-          if (parent == 0xffffffffu)
-            {
-              /* it's a root item */
-              name = gvdb_table_item_get_key (table, item, &name_length);
-
-              if (name != NULL)
-                {
-                  names[i] = g_strndup (name, name_length);
-                  filled++;
-                }
-            }
-
-          else if (parent < n_names && names[parent] != NULL)
-            {
-              /* It's a non-root item whose parent was filled in already.
-               *
-               * Calculate the name of this item by combining it with
-               * its parent name.
-               */
-              name = gvdb_table_item_get_key (table, item, &name_length);
-
-              if (name != NULL)
-                {
-                  const gchar *parent_name = names[parent];
-                  gsize parent_length;
-                  gchar *fullname;
-
-                  parent_length = strlen (parent_name);
-                  fullname = g_malloc (parent_length + name_length + 1);
-                  memcpy (fullname, parent_name, parent_length);
-                  memcpy (fullname + parent_length, name, name_length);
-                  fullname[parent_length + name_length] = '\0';
-                  names[i] = fullname;
-                  filled++;
-                }
-            }
-        }
-
-      total += filled;
-    }
-  while (filled && total < n_names);
-
-  /* If the table was corrupted then 'names' may have holes in it.
-   * Collapse those.
-   */
-  if G_UNLIKELY (total != n_names)
-    {
-      GPtrArray *fixed_names;
-
-      fixed_names = g_ptr_array_new ();
-      for (i = 0; i < n_names; i++)
-        if (names[i] != NULL)
-          g_ptr_array_add (fixed_names, names[i]);
-
-      g_free (names);
-      n_names = fixed_names->len;
-      g_ptr_array_add (fixed_names, NULL);
-      names = (gchar **) g_ptr_array_free (fixed_names, FALSE);
-    }
-
-  if (length)
-    *length = n_names;
-
-  return names;
-}
 
 /**
  * gvdb_table_list:
  * @file: a #GvdbTable
  * @key: a string
+ * @returns: a %NULL-terminated string array
  *
  * List all of the keys that appear below @key.  The nesting of keys
  * within the hash file is defined by the program that created the hash
@@ -495,8 +329,6 @@ gvdb_table_get_names (GvdbTable *table,
  *
  * You should call g_strfreev() on the return result when you no longer
  * require it.
- *
- * Returns: a %NULL-terminated string array
  **/
 gchar **
 gvdb_table_list (GvdbTable   *file,
@@ -547,27 +379,18 @@ gvdb_table_list (GvdbTable   *file,
  * gvdb_table_has_value:
  * @file: a #GvdbTable
  * @key: a string
+ * @returns: %TRUE if @key is in the table
  *
  * Checks for a value named @key in @file.
  *
  * Note: this function does not consider non-value nodes (other hash
  * tables, for example).
- *
- * Returns: %TRUE if @key is in the table
  **/
 gboolean
 gvdb_table_has_value (GvdbTable    *file,
                       const gchar  *key)
 {
-  static const struct gvdb_hash_item *item;
-  gsize size;
-
-  item = gvdb_table_lookup (file, key, 'v');
-
-  if (item == NULL)
-    return FALSE;
-
-  return gvdb_table_dereference (file, &item->value.pointer, 8, &size) != NULL;
+  return gvdb_table_lookup (file, key, 'v') != NULL;
 }
 
 static GVariant *
@@ -576,7 +399,6 @@ gvdb_table_value_from_item (GvdbTable                   *table,
 {
   GVariant *variant, *value;
   gconstpointer data;
-  GBytes *bytes;
   gsize size;
 
   data = gvdb_table_dereference (table, &item->value.pointer, 8, &size);
@@ -584,11 +406,12 @@ gvdb_table_value_from_item (GvdbTable                   *table,
   if G_UNLIKELY (data == NULL)
     return NULL;
 
-  bytes = g_bytes_new_from_bytes (table->bytes, ((gchar *) data) - table->data, size);
-  variant = g_variant_new_from_bytes (G_VARIANT_TYPE_VARIANT, bytes, table->trusted);
+  variant = g_variant_new_from_data (G_VARIANT_TYPE_VARIANT,
+                                     data, size, table->trusted,
+                                     (GDestroyNotify) g_mapped_file_unref,
+                                     g_mapped_file_ref (table->mapped));
   value = g_variant_get_variant (variant);
   g_variant_unref (variant);
-  g_bytes_unref (bytes);
 
   return value;
 }
@@ -597,6 +420,7 @@ gvdb_table_value_from_item (GvdbTable                   *table,
  * gvdb_table_get_value:
  * @file: a #GvdbTable
  * @key: a string
+ * @returns: a #GVariant, or %NULL
  *
  * Looks up a value named @key in @file.
  *
@@ -606,8 +430,6 @@ gvdb_table_value_from_item (GvdbTable                   *table,
  *
  * You should call g_variant_unref() on the return result when you no
  * longer require it.
- *
- * Returns: a #GVariant, or %NULL
  **/
 GVariant *
 gvdb_table_get_value (GvdbTable    *file,
@@ -637,13 +459,12 @@ gvdb_table_get_value (GvdbTable    *file,
  * gvdb_table_get_raw_value:
  * @table: a #GvdbTable
  * @key: a string
+ * @returns: a #GVariant, or %NULL
  *
  * Looks up a value named @key in @file.
  *
  * This call is equivalent to gvdb_table_get_value() except that it
  * never byteswaps the value.
- *
- * Returns: a #GVariant, or %NULL
  **/
 GVariant *
 gvdb_table_get_raw_value (GvdbTable   *table,
@@ -661,6 +482,7 @@ gvdb_table_get_raw_value (GvdbTable   *table,
  * gvdb_table_get_table:
  * @file: a #GvdbTable
  * @key: a string
+ * @returns: a new #GvdbTable, or %NULL
  *
  * Looks up the hash table named @key in @file.
  *
@@ -672,10 +494,8 @@ gvdb_table_get_raw_value (GvdbTable   *table,
  * contained in the file.  This newly-created #GvdbTable does not depend
  * on the continued existence of @file.
  *
- * You should call gvdb_table_free() on the return result when you no
+ * You should call gvdb_table_unref() on the return result when you no
  * longer require it.
- *
- * Returns: a new #GvdbTable, or %NULL
  **/
 GvdbTable *
 gvdb_table_get_table (GvdbTable   *file,
@@ -690,11 +510,12 @@ gvdb_table_get_table (GvdbTable   *file,
     return NULL;
 
   new = g_slice_new0 (GvdbTable);
-  new->bytes = g_bytes_ref (file->bytes);
+  new->mapped = g_mapped_file_ref (file->mapped);
   new->byteswapped = file->byteswapped;
   new->trusted = file->trusted;
   new->data = file->data;
   new->size = file->size;
+  new->ref_count = 1;
 
   gvdb_table_setup_root (new, &item->value.pointer);
 
@@ -702,32 +523,153 @@ gvdb_table_get_table (GvdbTable   *file,
 }
 
 /**
- * gvdb_table_free:
+ * gvdb_table_ref:
+ * @file: a #GvdbTable
+ * @returns: a new reference on @file
+ *
+ * Increases the reference count on @file.
+ **/
+GvdbTable *
+gvdb_table_ref (GvdbTable *file)
+{
+  g_atomic_int_inc (&file->ref_count);
+
+  return file;
+}
+
+/**
+ * gvdb_table_unref:
  * @file: a #GvdbTable
  *
- * Frees @file.
+ * Decreases the reference count on @file, possibly freeing it.
+ *
+ * Since: 2.26
  **/
 void
-gvdb_table_free (GvdbTable *file)
+gvdb_table_unref (GvdbTable *file)
 {
-  g_bytes_unref (file->bytes);
-  g_slice_free (GvdbTable, file);
+  if (g_atomic_int_dec_and_test (&file->ref_count))
+    {
+      g_mapped_file_unref (file->mapped);
+      g_slice_free (GvdbTable, file);
+    }
 }
 
 /**
  * gvdb_table_is_valid:
  * @table: a #GvdbTable
+ * @returns: %TRUE if @table is still valid
  *
  * Checks if the table is still valid.
  *
  * An on-disk GVDB can be marked as invalid.  This happens when the file
  * has been replaced.  The appropriate action is typically to reopen the
  * file.
- *
- * Returns: %TRUE if @table is still valid
  **/
 gboolean
 gvdb_table_is_valid (GvdbTable *table)
 {
   return !!*table->data;
 }
+
+/**
+ * gvdb_table_walk:
+ * @table: a #GvdbTable
+ * @key: a key corresponding to a list
+ * @open_func: the #GvdbWalkOpenFunc
+ * @value_func: the #GvdbWalkValueFunc
+ * @close_func: the #GvdbWalkCloseFunc
+ * @user_data: data to pass to the callbacks
+ *
+ * Looks up the list at @key and iterate over the items in it.
+ *
+ * First, @open_func is called to signal that we are starting to iterate over
+ * the list.  Then the list is iterated.  When all items in the list have been
+ * iterated over, the @close_func is called.
+ *
+ * When iterating, if a given item in the list is a value then @value_func is
+ * called.
+ *
+ * If a given item in the list is itself a list then @open_func is called.  If
+ * that function returns %TRUE then the walk begins iterating the items in the
+ * sublist, until there are no more items, at which point a matching
+ * @close_func call is made.  If @open_func returns %FALSE then no iteration of
+ * the sublist occurs and no corresponding @close_func call is made.
+ **/
+void
+gvdb_table_walk (GvdbTable         *table,
+                 const gchar       *key,
+                 GvdbWalkOpenFunc   open_func,
+                 GvdbWalkValueFunc  value_func,
+                 GvdbWalkCloseFunc  close_func,
+                 gpointer           user_data)
+{
+  const struct gvdb_hash_item *item;
+  const guint32_le *pointers[64];
+  const guint32_le *enders[64];
+  gsize name_lengths[64];
+  gint index = 0;
+
+  item = gvdb_table_lookup (table, key, 'L');
+  name_lengths[0] = 0;
+  pointers[0] = NULL;
+  enders[0] = NULL;
+  goto start_here;
+
+  while (index)
+    {
+      close_func (name_lengths[index], user_data);
+      index--;
+
+      while (pointers[index] < enders[index])
+        {
+          const gchar *name;
+          gsize name_len;
+
+          item = gvdb_table_get_item (table, *pointers[index]++);
+ start_here:
+
+          if (item != NULL &&
+              (name = gvdb_table_item_get_key (table, item, &name_len)))
+            {
+              if (item->type == 'L')
+                {
+                  if (open_func (name, name_len, user_data))
+                    {
+                      guint length = 0;
+
+                      index++;
+                      g_assert (index < 64);
+
+                      gvdb_table_list_from_item (table, item,
+                                                 &pointers[index],
+                                                 &length);
+                      enders[index] = pointers[index] + length;
+                      name_lengths[index] = name_len;
+                    }
+                }
+              else if (item->type == 'v')
+                {
+                  GVariant *value;
+
+                  value = gvdb_table_value_from_item (table, item);
+
+                  if (value != NULL)
+                    {
+                      if (table->byteswapped)
+                        {
+                          GVariant *tmp;
+
+                          tmp = g_variant_byteswap (value);
+                          g_variant_unref (value);
+                          value = tmp;
+                        }
+
+                      value_func (name, name_len, value, user_data);
+                      g_variant_unref (value);
+                    }
+                }
+            }
+        }
+    }
+}
