@@ -40,7 +40,7 @@ static GOptionEntry cmd_entries[] = {
    "Time out reads after the specified number of seconds", NULL},
   {"tls", 'T', 0, G_OPTION_ARG_STRING, &tls_cert_file,
    "Use TLS (SSL) with indicated server certificate", "CERTFILE"},
-  G_OPTION_ENTRY_NULL
+  {NULL}
 };
 
 #include "socket-common.c"
@@ -63,6 +63,8 @@ main (int argc,
   GInputStream *istream;
   GOutputStream *ostream;
 
+  g_type_init ();
+
   context = g_option_context_new (" - Test GSocket server stuff");
   g_option_context_add_main_entries (context, cmd_entries, NULL);
   if (!g_option_context_parse (context, &argc, &argv, &error))
@@ -356,7 +358,7 @@ main (int argc,
 
   if (!g_socket_close (socket, &error))
     {
-      g_printerr ("Error closing socket: %s\n",
+      g_printerr ("Error closing master socket: %s\n",
 		  error->message);
       return 1;
     }
