@@ -1,12 +1,10 @@
 /* GLIB - Library of useful routines for C programming
  * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
- * SPDX-License-Identifier: LGPL-2.1-or-later
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -53,36 +53,27 @@
 #include "gtestutils.h"
 #include "gthread.h"
 #include "gunicode.h"
-#include "gutilsprivate.h"
 
 #ifdef G_OS_WIN32
 #include "garray.h"
 #endif
 
 /**
- * GDate:
- * @julian_days: the Julian representation of the date
- * @julian: this bit is set if @julian_days is valid
- * @dmy: this is set if @day, @month and @year are valid
- * @day: the day of the day-month-year representation of the date,
- *   as a number between 1 and 31
- * @month: the month of the day-month-year representation of the date,
- *   as a number between 1 and 12
- * @year: the year of the day-month-year representation of the date
+ * SECTION:date
+ * @title: Date and Time Functions
+ * @short_description: calendrical calculations and miscellaneous time stuff
  *
- * `GDate` is a struct for calendrical calculations.
- *
- * The `GDate` data structure represents a day between January 1, Year 1,
+ * The #GDate data structure represents a day between January 1, Year 1,
  * and sometime a few thousand years in the future (right now it will go
- * to the year 65535 or so, but [method@GLib.Date.set_parse] only parses up to the
- * year 8000 or so - just count on "a few thousand"). `GDate` is meant to
+ * to the year 65535 or so, but g_date_set_parse() only parses up to the
+ * year 8000 or so - just count on "a few thousand"). #GDate is meant to
  * represent everyday dates, not astronomical dates or historical dates
  * or ISO timestamps or the like. It extrapolates the current Gregorian
  * calendar forward and backward in time; there is no attempt to change
- * the calendar to match time periods or locations. `GDate` does not store
- * time information; it represents a day.
+ * the calendar to match time periods or locations. #GDate does not store
+ * time information; it represents a <emphasis>day</emphasis>.
  *
- * The `GDate` implementation has several nice features; it is only a
+ * The #GDate implementation has several nice features; it is only a
  * 64-bit struct, so storing large numbers of dates is very efficient. It
  * can keep both a Julian and day-month-year representation of the date,
  * since some calculations are much easier with one representation or the
@@ -92,23 +83,25 @@
  * technical sense; technically, Julian dates count from the start of the
  * Julian period, Jan 1, 4713 BC).
  *
- * `GDate` is simple to use. First you need a "blank" date; you can get a
- * dynamically allocated date from [ctor@GLib.Date.new], or you can declare an
- * automatic variable or array and initialize it by calling [method@GLib.Date.clear].
- * A cleared date is safe; it's safe to call [method@GLib.Date.set_dmy] and the other
- * mutator functions to initialize the value of a cleared date. However, a cleared date
- * is initially invalid, meaning that it doesn't represent a day that exists.
- * It is undefined to call any of the date calculation routines on an invalid date.
- * If you obtain a date from a user or other unpredictable source, you should check
- * its validity with the [method@GLib.Date.valid] predicate. [method@GLib.Date.valid]
- * is also used to check for errors with [method@GLib.Date.set_parse] and other functions
- * that can fail. Dates can be invalidated by calling [method@GLib.Date.clear] again.
+ * #GDate is simple to use. First you need a "blank" date; you can get a
+ * dynamically allocated date from g_date_new(), or you can declare an
+ * automatic variable or array and initialize it to a sane state by
+ * calling g_date_clear(). A cleared date is sane; it's safe to call
+ * g_date_set_dmy() and the other mutator functions to initialize the
+ * value of a cleared date. However, a cleared date is initially
+ * <emphasis>invalid</emphasis>, meaning that it doesn't represent a day
+ * that exists. It is undefined to call any of the date calculation
+ * routines on an invalid date. If you obtain a date from a user or other
+ * unpredictable source, you should check its validity with the
+ * g_date_valid() predicate. g_date_valid() is also used to check for
+ * errors with g_date_set_parse() and other functions that can
+ * fail. Dates can be invalidated by calling g_date_clear() again.
  *
- * It is very important to use the API to access the `GDate` struct. Often only the
- * day-month-year or only the Julian representation is valid. Sometimes neither is valid.
- * Use the API.
+ * <emphasis>It is very important to use the API to access the #GDate
+ * struct.</emphasis> Often only the day-month-year or only the Julian
+ * representation is valid. Sometimes neither is valid. Use the API.
  *
- * GLib also features `GDateTime` which represents a precise time.
+ * GLib also features #GDateTime which represents a precise time.
  */
 
 /**
@@ -124,46 +117,56 @@
  * @tv_usec: microseconds
  *
  * Represents a precise time, with seconds and microseconds.
+ * Similar to the <structname>struct timeval</structname> returned by
+ * the gettimeofday() UNIX system call.
  *
- * Similar to the struct timeval returned by the `gettimeofday()`
- * UNIX system call.
- *
- * GLib is attempting to unify around the use of 64-bit integers to
+ * GLib is attempting to unify around the use of 64bit integers to
  * represent microsecond-precision time. As such, this type will be
- * removed from a future version of GLib. A consequence of using `glong` for
- * `tv_sec` is that on 32-bit systems `GTimeVal` is subject to the year 2038
- * problem.
+ * removed from a future version of GLib.
+ */
+
+/**
+ * GDate:
+ * @julian_days: the Julian representation of the date
+ * @julian: this bit is set if @julian_days is valid
+ * @dmy: this is set if @day, @month and @year are valid
+ * @day: the day of the day-month-year representation of the date,
+ *     as a number between 1 and 31
+ * @month: the day of the day-month-year representation of the date,
+ *     as a number between 1 and 12
+ * @year: the day of the day-month-year representation of the date
  *
- * Deprecated: 2.62: Use #GDateTime or #guint64 instead.
+ * Represents a day between January 1, Year 1 and a few thousand years in
+ * the future. None of its members should be accessed directly. If the
+ * <structname>GDate</structname> is obtained from g_date_new(), it will
+ * be safe to mutate but invalid and thus not safe for calendrical
+ * computations. If it's declared on the stack, it will contain garbage
+ * so must be initialized with g_date_clear(). g_date_clear() makes the
+ * date invalid but sane. An invalid date doesn't represent a day, it's
+ * "empty." A date becomes valid after you set it to a Julian day or you
+ * set a day, month, and year.
  */
 
 /**
  * GTime:
  *
- * Simply a replacement for `time_t`. It has been deprecated
- * since it is not equivalent to `time_t` on 64-bit platforms
- * with a 64-bit `time_t`.
- *
+ * Simply a replacement for <type>time_t</type>. It has been deprecated
+ * since it is <emphasis>not</emphasis> equivalent to <type>time_t</type>
+ * on 64-bit platforms with a 64-bit <type>time_t</type>.
  * Unrelated to #GTimer.
  *
- * Note that #GTime is defined to always be a 32-bit integer,
- * unlike `time_t` which may be 64-bit on some systems. Therefore,
- * #GTime will overflow in the year 2038, and you cannot use the
- * address of a #GTime variable as argument to the UNIX time()
- * function.
- *
- * Instead, do the following:
- *
- * |[<!-- language="C" -->
+ * Note that <type>GTime</type> is defined to always be a 32bit integer,
+ * unlike <type>time_t</type> which may be 64bit on some systems.
+ * Therefore, <type>GTime</type> will overflow in the year 2038, and
+ * you cannot use the address of a <type>GTime</type> variable as argument
+ * to the UNIX time() function. Instead, do the following:
+ * |[
  * time_t ttime;
  * GTime gtime;
  *
- * time (&ttime);
+ * time (&amp;ttime);
  * gtime = (GTime)ttime;
  * ]|
- *
- * Deprecated: 2.62: This is not [Y2038-safe](https://en.wikipedia.org/wiki/Year_2038_problem).
- *    Use #GDateTime or #time_t instead.
  */
 
 /**
@@ -179,9 +182,8 @@
 /**
  * GDateDay:
  *
- * Integer representing a day of the month; between 1 and 31.
- *
- * The %G_DATE_BAD_DAY value represents an invalid day of the month.
+ * Integer representing a day of the month; between 1 and
+ * 31. #G_DATE_BAD_DAY represents an invalid day of the month.
  */
 
 /**
@@ -200,20 +202,16 @@
  * @G_DATE_NOVEMBER: November
  * @G_DATE_DECEMBER: December
  *
- * Enumeration representing a month; values are %G_DATE_JANUARY,
- * %G_DATE_FEBRUARY, etc. %G_DATE_BAD_MONTH is the invalid value.
+ * Enumeration representing a month; values are #G_DATE_JANUARY,
+ * #G_DATE_FEBRUARY, etc. #G_DATE_BAD_MONTH is the invalid value.
  */
 
 /**
  * GDateYear:
  *
- * Integer type representing a year.
- *
- * The %G_DATE_BAD_YEAR value is the invalid value. The year
- * must be 1 or higher; negative ([BCE](https://en.wikipedia.org/wiki/Common_Era))
- * years are not allowed.
- *
- * The year is represented with four digits.
+ * Integer representing a year; #G_DATE_BAD_YEAR is the invalid
+ * value. The year must be 1 or higher; negative (BC) years are not
+ * allowed. The year is represented with four digits.
  */
 
 /**
@@ -227,8 +225,8 @@
  * @G_DATE_SATURDAY: Saturday
  * @G_DATE_SUNDAY: Sunday
  *
- * Enumeration representing a day of the week; %G_DATE_MONDAY,
- * %G_DATE_TUESDAY, etc. %G_DATE_BAD_WEEKDAY is an invalid weekday.
+ * Enumeration representing a day of the week; #G_DATE_MONDAY,
+ * #G_DATE_TUESDAY, etc. #G_DATE_BAD_WEEKDAY is an invalid weekday.
  */
 
 /**
@@ -253,7 +251,7 @@
  * g_date_new:
  *
  * Allocates a #GDate and initializes
- * it to a safe state. The new date will
+ * it to a sane state. The new date will
  * be cleared (as if you'd called g_date_clear()) but invalid (it won't
  * represent an existing day). Free the return value with g_date_free().
  *
@@ -273,14 +271,11 @@ g_date_new (void)
  * @month: month of the year
  * @year: year
  *
- * Create a new #GDate representing the given day-month-year triplet.
+ * Like g_date_new(), but also sets the value of the date. Assuming the
+ * day-month-year triplet you pass in represents an existing day, the
+ * returned date will be valid.
  *
- * The triplet you pass in must represent a valid date. Use g_date_valid_dmy()
- * if needed to validate it. The returned #GDate is guaranteed to be non-%NULL
- * and valid.
- *
- * Returns: (transfer full) (not nullable): a newly-allocated #GDate
- *   initialized with @day, @month, and @year
+ * Returns: a newly-allocated #GDate initialized with @day, @month, and @year
  */
 GDate*
 g_date_new_dmy (GDateDay   day, 
@@ -308,14 +303,11 @@ g_date_new_dmy (GDateDay   day,
  * g_date_new_julian:
  * @julian_day: days since January 1, Year 1
  *
- * Create a new #GDate representing the given Julian date.
+ * Like g_date_new(), but also sets the value of the date. Assuming the
+ * Julian day number you pass in is valid (greater than 0, less than an
+ * unreasonably large number), the returned date will be valid.
  *
- * The @julian_day you pass in must be valid. Use g_date_valid_julian() if
- * needed to validate it. The returned #GDate is guaranteed to be non-%NULL and
- * valid.
- *
- * Returns: (transfer full) (not nullable): a newly-allocated #GDate initialized
- *   with @julian_day
+ * Returns: a newly-allocated #GDate initialized with @julian_day
  */
 GDate*
 g_date_new_julian (guint32 julian_day)
@@ -349,35 +341,6 @@ g_date_free (GDate *date)
   g_free (date);
 }
 
-/**
- * g_date_copy:
- * @date: a #GDate to copy
- *
- * Copies a GDate to a newly-allocated GDate. If the input was invalid
- * (as determined by g_date_valid()), the invalid state will be copied
- * as is into the new object.
- *
- * Returns: (transfer full): a newly-allocated #GDate initialized from @date
- *
- * Since: 2.56
- */
-GDate *
-g_date_copy (const GDate *date)
-{
-  GDate *res;
-  g_return_val_if_fail (date != NULL, NULL);
-
-  if (g_date_valid (date))
-    res = g_date_new_julian (g_date_get_julian (date));
-  else
-    {
-      res = g_date_new ();
-      *res = *date;
-    }
-
-  return res;
-}
-
 /**
  * g_date_valid:
  * @date: a #GDate to check
@@ -420,7 +383,7 @@ static const guint16 days_in_year[2][14] =
 gboolean     
 g_date_valid_month (GDateMonth m)
 { 
-  return (((gint) m > G_DATE_BAD_MONTH) && ((gint) m < 13));
+  return ( (m > G_DATE_BAD_MONTH) && (m < 13) );
 }
 
 /**
@@ -466,7 +429,7 @@ g_date_valid_day (GDateDay d)
 gboolean     
 g_date_valid_weekday (GDateWeekday w)
 {
-  return (((gint) w > G_DATE_BAD_WEEKDAY) && ((gint) w < 8));
+  return ( (w > G_DATE_BAD_WEEKDAY) && (w < 8) );
 }
 
 /**
@@ -501,8 +464,6 @@ g_date_valid_dmy (GDateDay   d,
                   GDateMonth m, 
 		  GDateYear  y)
 {
-  /* No need to check the upper bound of @y, because #GDateYear is 16 bits wide,
-   * just like #GDate.year. */
   return ( (m > G_DATE_BAD_MONTH) &&
            (m < 13)               && 
            (d > G_DATE_BAD_DAY)   && 
@@ -523,7 +484,7 @@ g_date_update_julian (const GDate *const_d)
   gint idx;
   
   g_return_if_fail (d != NULL);
-  g_return_if_fail (d->dmy != 0);
+  g_return_if_fail (d->dmy);
   g_return_if_fail (!d->julian);
   g_return_if_fail (g_date_valid_dmy (d->day, d->month, d->year));
   
@@ -586,7 +547,7 @@ g_date_update_dmy (const GDate *const_d)
   
 #ifdef G_ENABLE_DEBUG
   if (!g_date_valid_dmy (day, m, y)) 
-    g_warning ("OOPS julian: %u  computed dmy: %u %u %u",
+    g_warning ("\nOOPS julian: %u  computed dmy: %u %u %u\n", 
 	       d->julian_days, day, m, y);
 #endif
   
@@ -736,8 +697,8 @@ g_date_get_day_of_year (const GDate *d)
  * @date: a #GDate
  *
  * Returns the week of the year, where weeks are understood to start on
- * Monday. If the date is before the first Monday of the year, return 0.
- * The date must be valid.
+ * Monday. If the date is before the first Monday of the year, return
+ * 0. The date must be valid.
  *
  * Returns: week of the year
  */
@@ -769,9 +730,9 @@ g_date_get_monday_week_of_year (const GDate *d)
  * g_date_get_sunday_week_of_year:
  * @date: a #GDate
  *
- * Returns the week of the year during which this date falls, if
- * weeks are understood to begin on Sunday. The date must be valid.
- * Can return 0 if the day is before the first Sunday of the year.
+ * Returns the week of the year during which this date falls, if weeks
+ * are understood to being on Sunday. The date must be valid. Can return
+ * 0 if the day is before the first Sunday of the year.
  *
  * Returns: week number
  */
@@ -862,7 +823,7 @@ g_date_days_between (const GDate *d1,
  * @date: pointer to one or more dates to clear
  * @n_dates: number of dates to clear
  *
- * Initializes one or more #GDate structs to a safe but invalid
+ * Initializes one or more #GDate structs to a sane but invalid
  * state. The cleared dates will not represent an existing date, but will
  * not contain garbage. Useful to init a date declared on the stack.
  * Validity can be tested with g_date_valid().
@@ -888,21 +849,11 @@ static gchar *long_month_names[13] =
   NULL,
 };
 
-static gchar *long_month_names_alternative[13] =
-{
-  NULL,
-};
-
 static gchar *short_month_names[13] = 
 {
   NULL, 
 };
 
-static gchar *short_month_names_alternative[13] =
-{
-  NULL,
-};
-
 /* This tells us if we need to update the parse info */
 static gchar *current_locale = NULL;
 
@@ -936,27 +887,6 @@ struct _GDateParseTokens {
 
 typedef struct _GDateParseTokens GDateParseTokens;
 
-static inline gboolean
-update_month_match (gsize *longest,
-                    const gchar *haystack,
-                    const gchar *needle)
-{
-  gsize length;
-
-  if (needle == NULL)
-    return FALSE;
-
-  length = strlen (needle);
-  if (*longest >= length)
-    return FALSE;
-
-  if (strstr (haystack, needle) == NULL)
-    return FALSE;
-
-  *longest = length;
-  return TRUE;
-}
-
 #define NUM_LEN 10
 
 /* HOLDS: g_date_global_lock */
@@ -1004,7 +934,6 @@ g_date_fill_parse_tokens (const gchar *str, GDateParseTokens *pt)
   
   if (pt->num_ints < 3)
     {
-      gsize longest = 0;
       gchar *casefold;
       gchar *normalized;
       
@@ -1012,37 +941,32 @@ g_date_fill_parse_tokens (const gchar *str, GDateParseTokens *pt)
       normalized = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
       g_free (casefold);
 
-      for (i = 1; i < 13; ++i)
+      i = 1;
+      while (i < 13)
         {
-          /* Here month names may be in a genitive case if the language
-           * grammatical rules require it.
-           * Examples of how January may look in some languages:
-           * Catalan: "de gener", Croatian: "siječnja", Polish: "stycznia",
-           * Upper Sorbian: "januara".
-           * Note that most of the languages can't or don't use the the
-           * genitive case here so they use nominative everywhere.
-           * For example, English always uses "January".
-           */
-          if (update_month_match (&longest, normalized, long_month_names[i]))
-            pt->month = i;
+          if (long_month_names[i] != NULL) 
+            {
+              const gchar *found = strstr (normalized, long_month_names[i]);
+	      
+              if (found != NULL)
+                {
+                  pt->month = i;
+		  break;
+                }
+            }
+	  
+          if (short_month_names[i] != NULL) 
+            {
+              const gchar *found = strstr (normalized, short_month_names[i]);
+	      
+              if (found != NULL)
+                {
+                  pt->month = i;
+		  break;
+                }
+            }
 
-          /* Here month names will be in a nominative case.
-           * Examples of how January may look in some languages:
-           * Catalan: "gener", Croatian: "Siječanj", Polish: "styczeń",
-           * Upper Sorbian: "Januar".
-           */
-          if (update_month_match (&longest, normalized, long_month_names_alternative[i]))
-            pt->month = i;
-
-          /* Differences between abbreviated nominative and abbreviated
-           * genitive month names are visible in very few languages but
-           * let's handle them.
-           */
-          if (update_month_match (&longest, normalized, short_month_names[i]))
-            pt->month = i;
-
-          if (update_month_match (&longest, normalized, short_month_names_alternative[i]))
-            pt->month = i;
+          ++i;
         }
 
       g_free (normalized);
@@ -1082,7 +1006,7 @@ g_date_prepare_to_parse (const gchar      *str,
         {
 	  gchar *casefold;
 	  
-          g_date_set_dmy (&d, 1, i, 1976);
+          g_date_set_dmy (&d, 1, i, 1);
 	  
           g_return_if_fail (g_date_valid (&d));
 	  
@@ -1099,18 +1023,6 @@ g_date_prepare_to_parse (const gchar      *str,
           long_month_names[i] = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
 	  g_free (casefold);
           
-          g_date_strftime (buf, 127, "%Ob", &d);
-          casefold = g_utf8_casefold (buf, -1);
-          g_free (short_month_names_alternative[i]);
-          short_month_names_alternative[i] = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
-          g_free (casefold);
-
-          g_date_strftime (buf, 127, "%OB", &d);
-          casefold = g_utf8_casefold (buf, -1);
-          g_free (long_month_names_alternative[i]);
-          long_month_names_alternative[i] = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
-          g_free (casefold);
-
           ++i;
         }
       
@@ -1123,12 +1035,6 @@ g_date_prepare_to_parse (const gchar      *str,
       g_date_strftime (buf, 127, "%x", &d);
       
       g_date_fill_parse_tokens (buf, &testpt);
-
-      using_twodigit_years = FALSE;
-      locale_era_adjust = 0;
-      dmy_order[0] = G_DATE_DAY;
-      dmy_order[1] = G_DATE_MONTH;
-      dmy_order[2] = G_DATE_YEAR;
       
       i = 0;
       while (i < testpt.num_ints)
@@ -1142,8 +1048,7 @@ g_date_prepare_to_parse (const gchar      *str,
               dmy_order[i] = G_DATE_DAY;
               break;
             case 76:
-              using_twodigit_years = TRUE;
-              G_GNUC_FALLTHROUGH;
+              using_twodigit_years = TRUE; /* FALL THRU */
             case 1976:
               dmy_order[i] = G_DATE_YEAR;
               break;
@@ -1164,13 +1069,6 @@ g_date_prepare_to_parse (const gchar      *str,
           DEBUG_MSG (("  %s   %s", long_month_names[i], short_month_names[i]));
           ++i;
         }
-      DEBUG_MSG (("Alternative month names:"));
-      i = 1;
-      while (i < 13)
-        {
-          DEBUG_MSG (("  %s   %s", long_month_names_alternative[i], short_month_names_alternative[i]));
-          ++i;
-        }
       if (using_twodigit_years)
         {
 	  DEBUG_MSG (("**Using twodigit years with cutoff year: %u", twodigit_start_year));
@@ -1198,7 +1096,7 @@ g_date_prepare_to_parse (const gchar      *str,
             ++i;
           }
         DEBUG_MSG (("**Order: %s, %s, %s", strings[0], strings[1], strings[2]));
-        DEBUG_MSG (("**Sample date in this locale: '%s'", buf));
+        DEBUG_MSG (("**Sample date in this locale: `%s'", buf));
       }
 #endif
     }
@@ -1206,29 +1104,13 @@ g_date_prepare_to_parse (const gchar      *str,
   g_date_fill_parse_tokens (str, pt);
 }
 
-static guint
-convert_twodigit_year (guint y)
-{
-  if (using_twodigit_years && y < 100)
-    {
-      guint two     =  twodigit_start_year % 100;
-      guint century = (twodigit_start_year / 100) * 100;
-
-      if (y < two)
-        century += 100;
-
-      y += century;
-    }
-  return y;
-}
-
 /**
  * g_date_set_parse:
  * @date: a #GDate to fill in
  * @str: string to parse
  *
  * Parses a user-inputted string @str, and try to figure out what date it
- * represents, taking the [current locale](running.html#locale)
+ * represents, taking the <link linkend="setlocale">current locale</link>
  * into account. If the string is successfully parsed, the date will be
  * valid after the call. Otherwise, it will be invalid. You should check
  * using g_date_valid() to see whether the parsing succeeded.
@@ -1245,28 +1127,17 @@ g_date_set_parse (GDate       *d,
 {
   GDateParseTokens pt;
   guint m = G_DATE_BAD_MONTH, day = G_DATE_BAD_DAY, y = G_DATE_BAD_YEAR;
-  gsize str_len;
   
   g_return_if_fail (d != NULL);
   
   /* set invalid */
   g_date_clear (d, 1);
-
-  /* Anything longer than this is ridiculous and could take a while to normalize.
-   * This limit is chosen arbitrarily. */
-  str_len = strlen (str);
-  if (str_len > 200)
-    return;
-
-  /* The input has to be valid UTF-8. */
-  if (!g_utf8_validate_len (str, str_len, NULL))
-    return;
-
+  
   G_LOCK (g_date_global);
 
   g_date_prepare_to_parse (str, &pt);
   
-  DEBUG_MSG (("Found %d ints, '%d' '%d' '%d' and written out month %d",
+  DEBUG_MSG (("Found %d ints, `%d' `%d' `%d' and written out month %d", 
 	      pt.num_ints, pt.n[0], pt.n[1], pt.n[2], pt.month));
   
   
@@ -1318,8 +1189,16 @@ g_date_set_parse (GDate       *d,
 	        {
 		  y += locale_era_adjust;
 	        }
-
-	      y = convert_twodigit_year (y);
+	      else if (using_twodigit_years && y < 100)
+		{
+		  guint two     =  twodigit_start_year % 100;
+		  guint century = (twodigit_start_year / 100) * 100;
+		  
+		  if (y < two)
+		    century += 100;
+		  
+		  y += century;
+		}
 	    }
 	    break;
             default:
@@ -1363,8 +1242,18 @@ g_date_set_parse (GDate       *d,
           m   = (pt.n[0]/100) % 100;
           day = pt.n[0] % 100;
           y   = pt.n[0]/10000;
-
-          y   = convert_twodigit_year (y);
+	  
+          /* FIXME move this into a separate function */
+          if (using_twodigit_years && y < 100)
+            {
+              guint two     =  twodigit_start_year % 100;
+              guint century = (twodigit_start_year / 100) * 100;
+              
+              if (y < two)
+                century += 100;
+              
+              y += century;
+            }
         }
     }
   
@@ -1386,49 +1275,18 @@ g_date_set_parse (GDate       *d,
   G_UNLOCK (g_date_global);
 }
 
-gboolean
-_g_localtime (time_t timet, struct tm *out_tm)
-{
-  gboolean success = TRUE;
-
-#ifdef HAVE_LOCALTIME_R
-  tzset ();
-  if (!localtime_r (&timet, out_tm))
-    success = FALSE;
-#else
-  {
-    struct tm *ptm = localtime (&timet);
-
-    if (ptm == NULL)
-      {
-        /* Happens at least in Microsoft's C library if you pass a
-         * negative time_t.
-         */
-        success = FALSE;
-      }
-    else
-      memcpy (out_tm, ptm, sizeof (struct tm));
-  }
-#endif
-
-  return success;
-}
-
 /**
  * g_date_set_time_t:
  * @date: a #GDate 
- * @timet: time_t value to set
+ * @timet: <type>time_t</type> value to set
  *
  * Sets the value of a date to the date corresponding to a time 
  * specified as a time_t. The time to date conversion is done using 
  * the user's current timezone.
  *
  * To set the value of a date to the current day, you could write:
- * |[<!-- language="C" -->
- *  time_t now = time (NULL);
- *  if (now == (time_t) -1)
- *    // handle the error
- *  g_date_set_time_t (date, now);
+ * |[
+ *  g_date_set_time_t (date, time (NULL)); 
  * ]|
  *
  * Since: 2.10
@@ -1438,21 +1296,33 @@ g_date_set_time_t (GDate *date,
 		   time_t timet)
 {
   struct tm tm;
-  gboolean success;
-
+  
   g_return_if_fail (date != NULL);
+  
+#ifdef HAVE_LOCALTIME_R
+  localtime_r (&timet, &tm);
+#else
+  {
+    struct tm *ptm = localtime (&timet);
 
-  success = _g_localtime (timet, &tm);
-  if (!success)
-    {
-      /* Still set a default date, 2000-01-01.
-       *
-       * We may assert out below. */
-      tm.tm_mon = 0;
-      tm.tm_mday = 1;
-      tm.tm_year = 100;
-    }
+    if (ptm == NULL)
+      {
+	/* Happens at least in Microsoft's C library if you pass a
+	 * negative time_t. Use 2000-01-01 as default date.
+	 */
+#ifndef G_DISABLE_CHECKS
+	g_return_if_fail_warning (G_LOG_DOMAIN, "g_date_set_time", "ptm != NULL");
+#endif
 
+	tm.tm_mon = 0;
+	tm.tm_mday = 1;
+	tm.tm_year = 100;
+      }
+    else
+      memcpy ((void *) &tm, (void *) ptm, sizeof(struct tm));
+  }
+#endif
+  
   date->julian = FALSE;
   
   date->month = tm.tm_mon + 1;
@@ -1462,11 +1332,6 @@ g_date_set_time_t (GDate *date,
   g_return_if_fail (g_date_valid_dmy (date->day, date->month, date->year));
   
   date->dmy    = TRUE;
-
-#ifndef G_DISABLE_CHECKS
-  if (!success)
-    g_return_if_fail_warning (G_LOG_DOMAIN, "g_date_set_time", "localtime() == NULL");
-#endif
 }
 
 
@@ -1480,14 +1345,12 @@ g_date_set_time_t (GDate *date,
  *
  * Deprecated: 2.10: Use g_date_set_time_t() instead.
  */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 void
 g_date_set_time (GDate *date,
 		 GTime  time_)
 {
   g_date_set_time_t (date, (time_t) time_);
 }
-G_GNUC_END_IGNORE_DEPRECATIONS
 
 /**
  * g_date_set_time_val:
@@ -1501,17 +1364,13 @@ G_GNUC_END_IGNORE_DEPRECATIONS
  * The time to date conversion is done using the user's current timezone.
  *
  * Since: 2.10
- * Deprecated: 2.62: #GTimeVal is not year-2038-safe. Use g_date_set_time_t()
- *    instead.
  */
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 void
 g_date_set_time_val (GDate    *date,
 		     GTimeVal *timeval)
 {
   g_date_set_time_t (date, (time_t) timeval->tv_sec);
 }
-G_GNUC_END_IGNORE_DEPRECATIONS
 
 /**
  * g_date_set_month:
@@ -1709,7 +1568,6 @@ g_date_add_days (GDate *d,
     g_date_update_julian (d);
 
   g_return_if_fail (d->julian);
-  g_return_if_fail (ndays <= G_MAXUINT32 - d->julian_days);
   
   d->julian_days += ndays;
   d->dmy = FALSE;
@@ -1763,16 +1621,13 @@ g_date_add_months (GDate *d,
   if (!d->dmy) 
     g_date_update_dmy (d);
 
-  g_return_if_fail (d->dmy != 0);
-  g_return_if_fail (nmonths <= G_MAXUINT - (d->month - 1));
-
+  g_return_if_fail (d->dmy);  
+  
   nmonths += d->month - 1;
   
   years  = nmonths/12;
   months = nmonths%12;
-
-  g_return_if_fail (years <= (guint) (G_MAXUINT16 - d->year));
-
+  
   d->month = months + 1;
   d->year  += years;
   
@@ -1808,7 +1663,7 @@ g_date_subtract_months (GDate *d,
   if (!d->dmy) 
     g_date_update_dmy (d);
 
-  g_return_if_fail (d->dmy != 0);
+  g_return_if_fail (d->dmy);  
   
   years  = nmonths/12;
   months = nmonths%12;
@@ -1854,9 +1709,8 @@ g_date_add_years (GDate *d,
   if (!d->dmy) 
     g_date_update_dmy (d);
 
-  g_return_if_fail (d->dmy != 0);
-  g_return_if_fail (nyears <= (guint) (G_MAXUINT16 - d->year));
-
+  g_return_if_fail (d->dmy);  
+  
   d->year += nyears;
   
   if (d->month == 2 && d->day == 29)
@@ -1888,7 +1742,7 @@ g_date_subtract_years (GDate *d,
   if (!d->dmy) 
     g_date_update_dmy (d);
 
-  g_return_if_fail (d->dmy != 0);
+  g_return_if_fail (d->dmy);  
   g_return_if_fail (d->year > nyears);
   
   d->year -= nyears;
@@ -1907,11 +1761,11 @@ g_date_subtract_years (GDate *d,
  * @year: year to check
  *
  * Returns %TRUE if the year is a leap year.
- *
- * For the purposes of this function, leap year is every year
- * divisible by 4 unless that year is divisible by 100. If it
- * is divisible by 100 it would be a leap year only if that year
- * is also divisible by 400.
+ * <footnote><para>For the purposes of this function,
+ * leap year is every year divisible by 4 unless that year
+ * is divisible by 100. If it is divisible by 100 it would
+ * be a leap year only if that year is also divisible
+ * by 400.</para></footnote>
  *
  * Returns: %TRUE if the year is a leap year
  */
@@ -2025,7 +1879,7 @@ g_date_get_sunday_weeks_in_year (GDateYear year)
  * @lhs: first date to compare
  * @rhs: second date to compare
  *
- * qsort()-style comparison function for dates.
+ * qsort()-style comparsion function for dates.
  * Both dates must be valid.
  *
  * Returns: 0 for equal, less than zero if @lhs is less than @rhs,
@@ -2082,11 +1936,12 @@ g_date_compare (const GDate *lhs,
 
 /**
  * g_date_to_struct_tm:
- * @date: a #GDate to set the struct tm from
- * @tm: (not nullable): struct tm to fill
+ * @date: a #GDate to set the <structname>struct tm</structname> from
+ * @tm: <structname>struct tm</structname> to fill
  *
- * Fills in the date-related bits of a struct tm using the @date value.
- * Initializes the non-date parts with something safe but meaningless.
+ * Fills in the date-related bits of a <structname>struct tm</structname>
+ * using the @date value. Initializes the non-date parts with something
+ * sane but meaningless.
  */
 void        
 g_date_to_struct_tm (const GDate *d, 
@@ -2100,7 +1955,7 @@ g_date_to_struct_tm (const GDate *d,
   if (!d->dmy) 
     g_date_update_dmy (d);
 
-  g_return_if_fail (d->dmy != 0);
+  g_return_if_fail (d->dmy);
   
   /* zero all the irrelevant fields to be sure they're valid */
   
@@ -2184,63 +2039,6 @@ g_date_order (GDate *date1,
 }
 
 #ifdef G_OS_WIN32
-static gboolean
-append_month_name (GArray     *result,
-		   LCID        lcid,
-		   SYSTEMTIME *systemtime,
-		   gboolean    abbreviated,
-		   gboolean    alternative)
-{
-  int n;
-  WORD base;
-  LPCWSTR lpFormat;
-
-  if (alternative)
-    {
-      base = abbreviated ? LOCALE_SABBREVMONTHNAME1 : LOCALE_SMONTHNAME1;
-      n = GetLocaleInfoW (lcid, base + systemtime->wMonth - 1, NULL, 0);
-      if (n == 0)
-        return FALSE;
-
-      g_array_set_size (result, result->len + n);
-      if (GetLocaleInfoW (lcid, base + systemtime->wMonth - 1,
-                          ((wchar_t *) result->data) + result->len - n, n) != n)
-        return FALSE;
-
-      g_array_set_size (result, result->len - 1);
-    }
-  else
-    {
-      /* According to MSDN, this is the correct method to obtain
-       * the form of the month name used when formatting a full
-       * date; it must be a genitive case in some languages.
-       *
-       * (n == 0) indicates an error, whereas (n < 2) is something we’d never
-       * expect from the given format string, and would break the subsequent code.
-       */
-      lpFormat = abbreviated ? L"ddMMM" : L"ddMMMM";
-      n = GetDateFormatW (lcid, 0, systemtime, lpFormat, NULL, 0);
-      if (n < 2)
-        return FALSE;
-
-      g_array_set_size (result, result->len + n);
-      if (GetDateFormatW (lcid, 0, systemtime, lpFormat,
-                          ((wchar_t *) result->data) + result->len - n, n) != n)
-        return FALSE;
-
-      /* We have obtained a day number as two digits and the month name.
-       * Now let's get rid of those two digits: overwrite them with the
-       * month name.
-       */
-      memmove (((wchar_t *) result->data) + result->len - n,
-	       ((wchar_t *) result->data) + result->len - n + 2,
-	       (n - 2) * sizeof (wchar_t));
-      g_array_set_size (result, result->len - 3);
-    }
-
-  return TRUE;
-}
-
 static gsize
 win32_strftime_helper (const GDate     *d,
 		       const gchar     *format,
@@ -2254,12 +2052,11 @@ win32_strftime_helper (const GDate     *d,
   int n, k;
   GArray *result;
   const gchar *p;
-  gunichar c, modifier;
+  gunichar c;
   const wchar_t digits[] = L"0123456789";
   gchar *convbuf;
   glong convlen = 0;
   gsize retval;
-  size_t format_len = strlen (format);
 
   systemtime.wYear = tm->tm_year + 1900;
   systemtime.wMonth = tm->tm_mon + 1;
@@ -2271,8 +2068,7 @@ win32_strftime_helper (const GDate     *d,
   systemtime.wMilliseconds = 0;
   
   lcid = GetThreadLocale ();
-  result = g_array_sized_new (FALSE, FALSE, sizeof (wchar_t),
-                              (format_len <= 64) ? (guint) format_len * 2 : 128);
+  result = g_array_sized_new (FALSE, FALSE, sizeof (wchar_t), MAX (128, strlen (format) * 2));
 
   p = format;
   while (*p)
@@ -2288,21 +2084,17 @@ win32_strftime_helper (const GDate     *d,
 
 	      return 0;
 	    }
-
-	  modifier = '\0';
+	  
 	  c = g_utf8_get_char (p);
 	  if (c == 'E' || c == 'O')
 	    {
-	      /* "%OB", "%Ob", and "%Oh" are supported, ignore other modified
-	       * conversion specifiers for now.
-	       */
-	      modifier = c;
+	      /* Ignore modified conversion specifiers for now. */
 	      p = g_utf8_next_char (p);
 	      if (!*p)
 		{
 		  s[0] = '\0';
 		  g_array_free (result, TRUE);
-
+		  
 		  return 0;
 		}
 
@@ -2333,16 +2125,16 @@ win32_strftime_helper (const GDate     *d,
 	      break;
 	    case 'b':
 	    case 'h':
-              if (!append_month_name (result, lcid, &systemtime, TRUE, modifier == 'O'))
-                {
-                  /* Ignore the error; this placeholder will be replaced with nothing */
-                }
+	      n = GetLocaleInfoW (lcid, LOCALE_SABBREVMONTHNAME1+systemtime.wMonth-1, NULL, 0);
+	      g_array_set_size (result, result->len + n);
+	      GetLocaleInfoW (lcid, LOCALE_SABBREVMONTHNAME1+systemtime.wMonth-1, ((wchar_t *) result->data) + result->len - n, n);
+	      g_array_set_size (result, result->len - 1);
 	      break;
 	    case 'B':
-              if (!append_month_name (result, lcid, &systemtime, FALSE, modifier == 'O'))
-                {
-                  /* Ignore the error; this placeholder will be replaced with nothing */
-                }
+	      n = GetLocaleInfoW (lcid, LOCALE_SMONTHNAME1+systemtime.wMonth-1, NULL, 0);
+	      g_array_set_size (result, result->len + n);
+	      GetLocaleInfoW (lcid, LOCALE_SMONTHNAME1+systemtime.wMonth-1, ((wchar_t *) result->data) + result->len - n, n);
+	      g_array_set_size (result, result->len - 1);
 	      break;
 	    case 'c':
 	      n = GetDateFormatW (lcid, 0, &systemtime, NULL, NULL, 0);
@@ -2564,7 +2356,9 @@ win32_strftime_helper (const GDate     *d,
 	      break;
 	    case 'Z':
 	      n = GetTimeZoneInformation (&tzinfo);
-	      if (n == TIME_ZONE_ID_UNKNOWN || n == TIME_ZONE_ID_STANDARD)
+	      if (n == TIME_ZONE_ID_UNKNOWN)
+		;
+	      else if (n == TIME_ZONE_ID_STANDARD)
 		g_array_append_vals (result, tzinfo.StandardName, wcslen (tzinfo.StandardName));
 	      else if (n == TIME_ZONE_ID_DAYLIGHT)
 		g_array_append_vals (result, tzinfo.DaylightName, wcslen (tzinfo.DaylightName));
@@ -2600,8 +2394,7 @@ win32_strftime_helper (const GDate     *d,
       return 0;
     }
   
-  g_assert (convlen >= 0);
-  if ((gsize) convlen >= slen)
+  if (slen <= convlen)
     {
       /* Ensure only whole characters are copied into the buffer. */
       gchar *end = g_utf8_find_prev_char (convbuf, convbuf + slen);
@@ -2631,7 +2424,7 @@ win32_strftime_helper (const GDate     *d,
  * @date: valid #GDate
  *
  * Generates a printed representation of the date, in a
- * [locale](running.html#locale)-specific way.
+ * <link linkend="setlocale">locale</link>-specific way.
  * Works just like the platform's C library strftime() function,
  * but only accepts date-related formats; time-related formats
  * give undefined results. Date must be valid. Unlike strftime()
@@ -2644,11 +2437,8 @@ win32_strftime_helper (const GDate     *d,
  * make the \%F provided by the C99 strftime() work on Windows
  * where the C library only complies to C89.
  *
- * Returns: number of characters written to the buffer, or `0` if the buffer was too small
+ * Returns: number of characters written to the buffer, or 0 the buffer was too small
  */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-
 gsize     
 g_date_strftime (gchar       *s, 
                  gsize        slen, 
@@ -2688,7 +2478,7 @@ g_date_strftime (gchar       *s,
 
   if (error)
     {
-      g_warning (G_STRLOC "Error converting format to locale encoding: %s", error->message);
+      g_warning (G_STRLOC "Error converting format to locale encoding: %s\n", error->message);
       g_error_free (error);
 
       s[0] = '\0';
@@ -2713,7 +2503,7 @@ g_date_strftime (gchar       *s,
 
           if (tmpbufsize > 65536)
             {
-              g_warning (G_STRLOC "Maximum buffer size for g_date_strftime exceeded: giving up");
+              g_warning (G_STRLOC "Maximum buffer size for g_date_strftime exceeded: giving up\n");
               g_free (locale_format);
 
               s[0] = '\0';
@@ -2730,11 +2520,9 @@ g_date_strftime (gchar       *s,
 
   if (error)
     {
-      g_warning (G_STRLOC "Error converting results of strftime to UTF-8: %s", error->message);
+      g_warning (G_STRLOC "Error converting results of strftime to UTF-8: %s\n", error->message);
       g_error_free (error);
 
-      g_assert (convbuf == NULL);
-
       s[0] = '\0';
       return 0;
     }
@@ -2761,5 +2549,3 @@ g_date_strftime (gchar       *s,
   return retval;
 #endif
 }
-
-#pragma GCC diagnostic pop
