@@ -861,6 +861,33 @@ set_ref_hash_test (void)
 
 GHashTable *h;
 
+typedef struct {
+    gchar *string;
+    gboolean freed;
+} FakeFreeData;
+
+GPtrArray *fake_free_data;
+
+static void
+fake_free (gpointer dead)
+{
+  guint i;
+
+  for (i = 0; i < fake_free_data->len; i++)
+    {
+      FakeFreeData *ffd = g_ptr_array_index (fake_free_data, i);
+
+      if (ffd->string == (gchar *) dead)
+        {
+          g_assert (!ffd->freed);
+          ffd->freed = TRUE;
+          return;
+        }
+    }
+
+  g_assert_not_reached ();
+}
+
 static void
 value_destroy_insert (gpointer value)
 {
@@ -870,18 +897,59 @@ value_destroy_insert (gpointer value)
 static void
 test_destroy_modify (void)
 {
+  FakeFreeData *ffd;
+  guint i;
+
   g_test_bug ("650459");
 
-  h = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, value_destroy_insert);
+  fake_free_data = g_ptr_array_new ();
 
-  g_hash_table_insert (h, g_strdup ("a"), g_strdup ("b"));
-  g_hash_table_insert (h, g_strdup ("c"), g_strdup ("d"));
-  g_hash_table_insert (h, g_strdup ("e"), g_strdup ("f"));
-  g_hash_table_insert (h, g_strdup ("g"), g_strdup ("h"));
-  g_hash_table_insert (h, g_strdup ("h"), g_strdup ("k"));
-  g_hash_table_insert (h, g_strdup ("a"), g_strdup ("c"));
+  h = g_hash_table_new_full (g_str_hash, g_str_equal, fake_free, value_destroy_insert);
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("a");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "b");
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("c");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "d");
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("e");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "f");
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("g");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "h");
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("h");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "k");
+
+  ffd = g_new0 (FakeFreeData, 1);
+  ffd->string = g_strdup ("a");
+  g_ptr_array_add (fake_free_data, ffd);
+  g_hash_table_insert (h, ffd->string, "c");
 
   g_hash_table_remove (h, "c");
+
+  /* that removed everything... */
+  for (i = 0; i < fake_free_data->len; i++)
+    {
+      FakeFreeData *ffd = g_ptr_array_index (fake_free_data, i);
+
+      g_assert (ffd->freed);
+      g_free (ffd);
+    }
+
+  g_ptr_array_unref (fake_free_data);
+
+  /* ... so this is a no-op */
   g_hash_table_remove (h, "e");
 
   g_hash_table_unref (h);
@@ -1098,6 +1166,52 @@ test_internal_consistency (void)
   g_hash_table_unref (h);
 }
 
+static void
+my_key_free (gpointer v)
+{
+  gchar *s = v;
+  g_assert (s[0] != 'x');
+  s[0] = 'x';
+  g_free (v);
+}
+
+static void
+my_value_free (gpointer v)
+{
+  gchar *s = v;
+  g_assert (s[0] != 'y');
+  s[0] = 'y';
+  g_free (v);
+}
+
+static void
+test_iter_replace (void)
+{
+  GHashTable *h;
+  GHashTableIter iter;
+  gpointer k, v;
+  gchar *s;
+
+  g_test_bug ("662544");
+
+  h = g_hash_table_new_full (g_str_hash, g_str_equal, my_key_free, my_value_free);
+
+  g_hash_table_insert (h, g_strdup ("A"), g_strdup ("a"));
+  g_hash_table_insert (h, g_strdup ("B"), g_strdup ("b"));
+  g_hash_table_insert (h, g_strdup ("C"), g_strdup ("c"));
+
+  g_hash_table_iter_init (&iter, h);
+
+  while (g_hash_table_iter_next (&iter, &k, &v))
+    {
+       s = (gchar*)v;
+       g_assert (g_ascii_islower (s[0]));
+       g_hash_table_iter_replace (&iter, g_strdup (k));
+    }
+
+  g_hash_table_unref (h);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1123,6 +1237,7 @@ main (int argc, char *argv[])
   g_test_add_func ("/hash/lookup-null-key", test_lookup_null_key);
   g_test_add_func ("/hash/destroy-modify", test_destroy_modify);
   g_test_add_func ("/hash/consistency", test_internal_consistency);
+  g_test_add_func ("/hash/iter-replace", test_iter_replace);
 
   return g_test_run ();
 
