@@ -100,16 +100,37 @@ test_default_handler (void)
       exit (0);
     }
   g_test_trap_assert_passed ();
+  g_test_trap_assert_stdout_unmatched ("*INFO*message5*");
+
+  g_setenv ("G_MESSAGES_DEBUG", "foo bar baz", TRUE);
+  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT))
+    {
+      g_log ("bar", G_LOG_LEVEL_INFO, "message5");
+      exit (0);
+    }
+  g_test_trap_assert_passed ();
   g_test_trap_assert_stdout ("*INFO*message5*");
 
   if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT))
     {
-      g_debug ("message6");
+      g_log ("baz", G_LOG_LEVEL_DEBUG, "message6");
       exit (0);
     }
   g_test_trap_assert_passed ();
   g_test_trap_assert_stdout ("*DEBUG*message6*");
 
+  g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+  if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT))
+    {
+      g_log ("foo", G_LOG_LEVEL_DEBUG, "6");
+      g_log ("bar", G_LOG_LEVEL_DEBUG, "6");
+      g_log ("baz", G_LOG_LEVEL_DEBUG, "6");
+      exit (0);
+    }
+  g_test_trap_assert_passed ();
+  g_test_trap_assert_stdout ("*DEBUG*6*6*6*");
+
+  g_unsetenv ("G_MESSAGES_DEBUG");
   if (g_test_trap_fork (0, G_TEST_TRAP_SILENCE_STDOUT))
     {
       g_log (G_LOG_DOMAIN, 1<<10, "message7");
@@ -172,7 +193,7 @@ test_printerr_handler (void)
 static char *fail_str = "foo";
 static char *log_str = "bar";
 
-gboolean
+static gboolean
 good_failure_handler (const gchar    *log_domain,
                       GLogLevelFlags  log_level,
                       const gchar    *msg,
@@ -185,7 +206,7 @@ good_failure_handler (const gchar    *log_domain,
   return FALSE;
 }
 
-gboolean
+static gboolean
 bad_failure_handler (const gchar    *log_domain,
                      GLogLevelFlags  log_level,
                      const gchar    *msg,
@@ -198,7 +219,7 @@ bad_failure_handler (const gchar    *log_domain,
   return FALSE;
 }
 
-void
+static void
 test_handler (const gchar    *log_domain,
               GLogLevelFlags  log_level,
               const gchar    *msg,
@@ -209,7 +230,7 @@ test_handler (const gchar    *log_domain,
   g_assert ((char *)user_data == log_str);
 }
 
-void
+static void
 bug653052 (void)
 {
   g_test_bug ("653052");
