@@ -95,6 +95,7 @@ group_captions (void)
               g_setenv ("LANG", "C", TRUE);
 
               g_option_context_parse (options, &argc, &argv, &error);
+              g_option_context_free (options);
               exit(0);
             }
           else
@@ -161,6 +162,8 @@ group_captions (void)
                 g_test_trap_assert_stdout_unmatched ("*--help-test*");
             }
         }
+
+      g_option_context_free (options);
     }
 }
 
@@ -188,7 +191,7 @@ gboolean ignore_test1_boolean;
 gboolean ignore_test2_boolean;
 gchar *ignore_test3_string;
 
-gchar **
+static gchar **
 split_string (const char *str, int *argc)
 {
   gchar **argv;
@@ -204,7 +207,7 @@ split_string (const char *str, int *argc)
   return argv;
 }
 
-gchar *
+static gchar *
 join_stringv (int argc, char **argv)
 {
   int i;
@@ -224,12 +227,28 @@ join_stringv (int argc, char **argv)
 }
 
 /* Performs a shallow copy */
-char **
+static char **
 copy_stringv (char **argv, int argc)
 {
   return g_memdup (argv, sizeof (char *) * (argc + 1));
 }
 
+static void
+check_identical_stringv (gchar **before, gchar **after)
+{
+  guint i;
+
+  /* Not only is it the same string... */
+  for (i = 0; before[i] != NULL; i++)
+    g_assert_cmpstr (before[i], ==, after[i]);
+
+  /* ... it is actually the same pointer */
+  for (i = 0; before[i] != NULL; i++)
+    g_assert (before[i] == after[i]);
+
+  g_assert (after[i] == NULL);
+}
+
 
 static gboolean
 error_test1_pre_parse (GOptionContext *context,
@@ -256,13 +275,14 @@ error_test1_post_parse (GOptionContext *context,
   return FALSE;
 }
 
-void
+static void
 error_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionGroup *main_group;
   GOptionEntry entries [] =
@@ -281,14 +301,20 @@ error_test1 (void)
   
   /* Now try parsing */
   argv = split_string ("program --test 20", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  /* An error occurred, so argv has not been changed */
+  check_identical_stringv (argv_copy, argv);
+  g_clear_error (&error);
 
   /* On failure, values should be reset */
   g_assert (error_test1_int == 0x12345678);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -317,13 +343,14 @@ error_test2_post_parse (GOptionContext *context,
   return FALSE;
 }
 
-void
+static void
 error_test2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionGroup *main_group;
   GOptionEntry entries [] =
@@ -342,14 +369,18 @@ error_test2 (void)
   
   /* Now try parsing */
   argv = split_string ("program --test bar", &argc);
+  argv_copy = copy_stringv (argv, argc);
   retval = g_option_context_parse (context, &argc, &argv, &error);
 
-  g_error_free (error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  check_identical_stringv (argv_copy, argv);
+  g_clear_error (&error);
 
   g_assert (strcmp (error_test2_string, "foo") == 0);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -378,13 +409,14 @@ error_test3_post_parse (GOptionContext *context,
   return FALSE;
 }
 
-void
+static void
 error_test3 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionGroup *main_group;
   GOptionEntry entries [] =
@@ -403,24 +435,29 @@ error_test3 (void)
   
   /* Now try parsing */
   argv = split_string ("program --test", &argc);
+  argv_copy = copy_stringv (argv, argc);
   retval = g_option_context_parse (context, &argc, &argv, &error);
 
-  g_error_free (error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  check_identical_stringv (argv_copy, argv);
+  g_clear_error (&error);
 
   g_assert (!error_test3_boolean);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 arg_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_INT, &arg_test1_int, NULL, NULL },
@@ -431,6 +468,7 @@ arg_test1 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test 20 --test 30", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
@@ -439,17 +477,22 @@ arg_test1 (void)
   /* Last arg specified is the one that should be stored */
   g_assert (arg_test1_int == 30);
 
-  g_strfreev (argv);
+  /* We free all of the strings in a copy of argv, because now argv is a
+   * subset - some have been removed in-place
+   */
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 arg_test2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_STRING, &arg_test2_string, NULL, NULL },
@@ -460,7 +503,8 @@ arg_test2 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo --test bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -469,18 +513,20 @@ arg_test2 (void)
   g_assert (strcmp (arg_test2_string, "bar") == 0);
 
   g_free (arg_test2_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 arg_test3 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_FILENAME, &arg_test3_filename, NULL, NULL },
@@ -491,7 +537,8 @@ arg_test3 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -500,18 +547,20 @@ arg_test3 (void)
   g_assert (strcmp (arg_test3_filename, "foo.txt") == 0);
 
   g_free (arg_test3_filename);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 
-void
+static void
 arg_test4 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
+  gchar **argv_copy;
   gchar **argv;
   int argc;
   GOptionEntry entries [] =
@@ -523,6 +572,7 @@ arg_test4 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test 20.0 --test 30.03", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
@@ -531,17 +581,19 @@ arg_test4 (void)
   /* Last arg specified is the one that should be stored */
   g_assert (arg_test4_double == 30.03);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 arg_test5 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   char *old_locale, *current_locale;
   const char *locale = "de_DE";
@@ -554,6 +606,7 @@ arg_test5 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test 20,0 --test 30,03", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   /* set it to some locale that uses commas instead of decimal points */
   
@@ -576,17 +629,19 @@ arg_test5 (void)
   setlocale (LC_NUMERIC, old_locale);
   g_free (old_locale);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 arg_test6 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_INT64, &arg_test6_int64, NULL, NULL },
@@ -598,6 +653,7 @@ arg_test6 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test 4294967297 --test 4294967296 --test2 0xfffffffff", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
@@ -607,7 +663,8 @@ arg_test6 (void)
   g_assert (arg_test6_int64 == G_GINT64_CONSTANT(4294967296));
   g_assert (arg_test6_int64_2 == G_GINT64_CONSTANT(0xfffffffff));
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -619,13 +676,14 @@ callback_parse1 (const gchar *option_name, const gchar *value,
 	return TRUE;
 }
 
-void
+static void
 callback_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_CALLBACK, callback_parse1, NULL, NULL },
@@ -636,7 +694,8 @@ callback_test1 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -644,8 +703,9 @@ callback_test1 (void)
   g_assert (strcmp (callback_test1_string, "foo.txt") == 0);
 
   g_free (callback_test1_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -657,13 +717,14 @@ callback_parse2 (const gchar *option_name, const gchar *value,
 	return TRUE;
 }
 
-void
+static void
 callback_test2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, callback_parse2, NULL, NULL },
@@ -674,14 +735,16 @@ callback_test2 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test --test", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
 
   g_assert (callback_test2_int == 2);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -697,13 +760,14 @@ callback_parse_optional (const gchar *option_name, const gchar *value,
 	return TRUE;
 }
 
-void
+static void
 callback_test_optional_1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, 
@@ -715,7 +779,8 @@ callback_test_optional_1 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -725,18 +790,20 @@ callback_test_optional_1 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, 
@@ -748,7 +815,8 @@ callback_test_optional_2 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -758,17 +826,19 @@ callback_test_optional_2 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_3 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
+  gchar **argv_copy;
   gchar **argv;
   int argc;
   GOptionEntry entries [] =
@@ -781,7 +851,8 @@ callback_test_optional_3 (void)
 
   /* Now try parsing */
   argv = split_string ("program -t foo.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -791,19 +862,21 @@ callback_test_optional_3 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 
-void
+static void
 callback_test_optional_4 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 't', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, 
@@ -815,7 +888,8 @@ callback_test_optional_4 (void)
 
   /* Now try parsing */
   argv = split_string ("program -t", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -825,12 +899,13 @@ callback_test_optional_4 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_5 (void)
 {
   GOptionContext *context;
@@ -838,6 +913,7 @@ callback_test_optional_5 (void)
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "dummy", 'd', 0, G_OPTION_ARG_NONE, &dummy, NULL },
@@ -850,7 +926,8 @@ callback_test_optional_5 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test --dummy", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -860,12 +937,13 @@ callback_test_optional_5 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_6 (void)
 {
   GOptionContext *context;
@@ -873,6 +951,7 @@ callback_test_optional_6 (void)
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "dummy", 'd', 0, G_OPTION_ARG_NONE, &dummy, NULL },
@@ -885,7 +964,8 @@ callback_test_optional_6 (void)
 
   /* Now try parsing */
   argv = split_string ("program -t -d", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -895,12 +975,13 @@ callback_test_optional_6 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_7 (void)
 {
   GOptionContext *context;
@@ -908,6 +989,7 @@ callback_test_optional_7 (void)
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "dummy", 'd', 0, G_OPTION_ARG_NONE, &dummy, NULL },
@@ -920,7 +1002,8 @@ callback_test_optional_7 (void)
 
   /* Now try parsing */
   argv = split_string ("program -td", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -930,12 +1013,13 @@ callback_test_optional_7 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_8 (void)
 {
   GOptionContext *context;
@@ -943,6 +1027,7 @@ callback_test_optional_8 (void)
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "dummy", 'd', 0, G_OPTION_ARG_NONE, &dummy, NULL },
@@ -955,7 +1040,8 @@ callback_test_optional_8 (void)
 
   /* Now try parsing */
   argv = split_string ("program -dt foo.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -965,18 +1051,20 @@ callback_test_optional_8 (void)
   g_assert (callback_test_optional_boolean);
 
   g_free (callback_test_optional_string);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_9 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   gchar *string = NULL;
   GOptionEntry entries [] =
@@ -989,24 +1077,28 @@ callback_test_optional_9 (void)
 
   /* Now try parsing */
   argv = split_string ("program -t", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (!retval);
   g_assert (string == NULL);
+  check_identical_stringv (argv_copy, argv);
 
   g_error_free (error);
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 callback_test_optional_10 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   gchar *string = NULL;
   GOptionEntry entries [] =
@@ -1019,14 +1111,17 @@ callback_test_optional_10 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (!retval);
   g_assert (string == NULL);
+  check_identical_stringv (argv_copy, argv);
 
   g_error_free (error);
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -1039,13 +1134,14 @@ callback_remaining_test1_callback (const gchar *option_name, const gchar *value,
 	return TRUE;
 }
 
-void
+static void
 callback_remaining_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_CALLBACK, callback_remaining_test1_callback, NULL, NULL },
@@ -1057,7 +1153,8 @@ callback_remaining_test1 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo.txt blah.txt", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1068,8 +1165,9 @@ callback_remaining_test1 (void)
 
   g_ptr_array_foreach (callback_remaining_args, (GFunc) g_free, NULL);
   g_ptr_array_free (callback_remaining_args, TRUE);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -1088,6 +1186,7 @@ callback_returns_false (void)
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "error", 0, 0, G_OPTION_ARG_CALLBACK, callback_error, NULL, NULL },
@@ -1100,56 +1199,72 @@ callback_returns_false (void)
 
   /* Now try parsing */
   argv = split_string ("program --error value", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
+  check_identical_stringv (argv_copy, argv);
 
   g_option_context_free (context);
   g_clear_error (&error);
+  g_strfreev (argv_copy);
+  g_free (argv);
 
   /* And again, this time with a no-arg variant */
   context = g_option_context_new (NULL);
   g_option_context_add_main_entries (context, entries, NULL);
 
   argv = split_string ("program --error-no-arg", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
+  check_identical_stringv (argv_copy, argv);
 
   g_option_context_free (context);
   g_clear_error (&error);
+  g_strfreev (argv_copy);
+  g_free (argv);
 
   /* And again, this time with a optional arg variant, with argument */
   context = g_option_context_new (NULL);
   g_option_context_add_main_entries (context, entries, NULL);
 
   argv = split_string ("program --error-optional-arg value", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
+  check_identical_stringv (argv_copy, argv);
 
   g_option_context_free (context);
   g_clear_error (&error);
+  g_strfreev (argv_copy);
+  g_free (argv);
 
   /* And again, this time with a optional arg variant, without argument */
   context = g_option_context_new (NULL);
   g_option_context_add_main_entries (context, entries, NULL);
 
   argv = split_string ("program --error-optional-arg", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE);
   g_assert (retval == FALSE);
+  check_identical_stringv (argv_copy, argv);
 
   g_option_context_free (context);
   g_clear_error (&error);
+  g_strfreev (argv_copy);
+  g_free (argv);
 }
 
 
-void
+static void
 ignore_test1 (void)
 {
   GOptionContext *context;
@@ -1184,13 +1299,14 @@ ignore_test1 (void)
   g_option_context_free (context);
 }
 
-void
+static void
 ignore_test2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   gchar *arg;
   GOptionEntry entries [] =
@@ -1203,7 +1319,8 @@ ignore_test2 (void)
 
   /* Now try parsing */
   argv = split_string ("program -test", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1213,11 +1330,12 @@ ignore_test2 (void)
   g_assert (strcmp (arg, "program -es") == 0);
 
   g_free (arg);
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 ignore_test3 (void)
 {
   GOptionContext *context;
@@ -1256,12 +1374,13 @@ ignore_test3 (void)
 }
 
 void
-array_test1 (void)
+static array_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] =
     { { "test", 0, 0, G_OPTION_ARG_STRING_ARRAY, &array_test1_array, NULL, NULL },
@@ -1272,7 +1391,8 @@ array_test1 (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo --test bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1284,11 +1404,12 @@ array_test1 (void)
 
   g_strfreev (array_test1_array);
   
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 add_test1 (void)
 {
   GOptionContext *context;
@@ -1307,7 +1428,7 @@ add_test1 (void)
   g_option_context_free (context);
 }
 
-void
+static void
 empty_test2 (void)
 {
   GOptionContext *context;
@@ -1318,7 +1439,7 @@ empty_test2 (void)
   g_option_context_free (context);
 }
 
-void
+static void
 empty_test3 (void)
 {
   GOptionContext *context;
@@ -1335,13 +1456,14 @@ empty_test3 (void)
 }
 
 /* check that non-option arguments are left in argv by default */
-void
+static void
 rest_test1 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1353,7 +1475,8 @@ rest_test1 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1365,18 +1488,20 @@ rest_test1 (void)
   g_assert (strcmp (argv[2], "bar") == 0);
   g_assert (argv[3] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 /* check that -- works */
-void
+static void
 rest_test2 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1388,7 +1513,8 @@ rest_test2 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test -- -bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1401,18 +1527,20 @@ rest_test2 (void)
   g_assert (strcmp (argv[3], "-bar") == 0);
   g_assert (argv[4] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 /* check that -- stripping works */
-void
+static void
 rest_test2a (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1424,7 +1552,8 @@ rest_test2a (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test -- bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1436,17 +1565,19 @@ rest_test2a (void)
   g_assert (strcmp (argv[2], "bar") == 0);
   g_assert (argv[3] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 rest_test2b (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1459,7 +1590,8 @@ rest_test2b (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test -bar --", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1471,17 +1603,19 @@ rest_test2b (void)
   g_assert (strcmp (argv[2], "-bar") == 0);
   g_assert (argv[3] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 rest_test2c (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1493,7 +1627,8 @@ rest_test2c (void)
 
   /* Now try parsing */
   argv = split_string ("program --test foo -- bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1505,17 +1640,19 @@ rest_test2c (void)
   g_assert (strcmp (argv[2], "bar") == 0);
   g_assert (argv[3] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 rest_test2d (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1527,7 +1664,8 @@ rest_test2d (void)
 
   /* Now try parsing */
   argv = split_string ("program --test -- -bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1539,19 +1677,21 @@ rest_test2d (void)
   g_assert (strcmp (argv[2], "-bar") == 0);
   g_assert (argv[3] == NULL);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 
 /* check that G_OPTION_REMAINING collects non-option arguments */
-void
+static void
 rest_test3 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1564,7 +1704,8 @@ rest_test3 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1576,20 +1717,22 @@ rest_test3 (void)
   g_assert (array_test1_array[2] == NULL);
 
   g_strfreev (array_test1_array);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 
 /* check that G_OPTION_REMAINING and -- work together */
-void
+static void
 rest_test4 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1602,7 +1745,8 @@ rest_test4 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test -- -bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1614,19 +1758,21 @@ rest_test4 (void)
   g_assert (array_test1_array[2] == NULL);
 
   g_strfreev (array_test1_array);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 /* test that G_OPTION_REMAINING works with G_OPTION_ARG_FILENAME_ARRAY */
-void
+static void
 rest_test5 (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { 
       { "test", 0, 0, G_OPTION_ARG_NONE, &ignore_test1_boolean, NULL, NULL },
@@ -1639,7 +1785,8 @@ rest_test5 (void)
 
   /* Now try parsing */
   argv = split_string ("program foo --test bar", &argc);
-  
+  argv_copy = copy_stringv (argv, argc);
+
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
   g_assert (retval);
@@ -1651,18 +1798,20 @@ rest_test5 (void)
   g_assert (array_test1_array[2] == NULL);
 
   g_strfreev (array_test1_array);
-  
-  g_strfreev (argv);
+
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 unknown_short_test (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   GOptionEntry entries [] = { { NULL } };
 
@@ -1673,21 +1822,27 @@ unknown_short_test (void)
 
   /* Now try parsing */
   argv = split_string ("program -0", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert (!retval);
+  g_assert (error != NULL);
+  g_clear_error (&error);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
 /* test that lone dashes are treated as non-options */
-void lonely_dash_test (void)
+static void
+lonely_dash_test (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
 
   g_test_bug ("168008");
@@ -1696,6 +1851,7 @@ void lonely_dash_test (void)
 
   /* Now try parsing */
   argv = split_string ("program -", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert_no_error (error);
@@ -1703,17 +1859,19 @@ void lonely_dash_test (void)
 
   g_assert (argv[1] && strcmp (argv[1], "-") == 0);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
-void
+static void
 missing_arg_test (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   gchar *arg = NULL;
   GOptionEntry entries [] =
@@ -1727,20 +1885,31 @@ missing_arg_test (void)
 
   /* Now try parsing */
   argv = split_string ("program --test", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  /* An error occurred, so argv has not been changed */
+  check_identical_stringv (argv_copy, argv);
   g_clear_error (&error);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
 
   /* Try parsing again */
   argv = split_string ("program -t", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  /* An error occurred, so argv has not been changed */
+  check_identical_stringv (argv_copy, argv);
+  g_clear_error (&error);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
@@ -1755,13 +1924,14 @@ static gboolean cb (const gchar  *option_name,
   return TRUE;
 }
 
-void
+static void
 dash_arg_test (void)
 {
   GOptionContext *context;
   gboolean retval;
   GError *error = NULL;
   gchar **argv;
+  gchar **argv_copy;
   int argc;
   gboolean argb = FALSE;
   GOptionEntry entries [] =
@@ -1776,6 +1946,7 @@ dash_arg_test (void)
 
   /* Now try parsing */
   argv = split_string ("program --test=-3", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   test_arg = NULL;
   error = NULL;
@@ -1784,12 +1955,14 @@ dash_arg_test (void)
   g_assert_no_error (error);
   g_assert_cmpstr (test_arg, ==, "-3");
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_free (test_arg);
   test_arg = NULL;
 
   /* Try parsing again */
   argv = split_string ("program --test -3", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   error = NULL;
   retval = g_option_context_parse (context, &argc, &argv, &error);
@@ -1798,6 +1971,8 @@ dash_arg_test (void)
   g_assert_cmpstr (test_arg, ==, NULL);
 
   g_option_context_free (context);
+  g_strfreev (argv_copy);
+  g_free (argv);
 }
 
 static void
@@ -1870,6 +2045,7 @@ test_error_hook (void)
       { NULL } };
   GOptionGroup *group;
   gchar **argv;
+  gchar **argv_copy;
   gint argc;
   gboolean retval;
   GError *error = NULL;
@@ -1882,14 +2058,19 @@ test_error_hook (void)
                                  error_func);
 
   argv = split_string ("program --test", &argc);
+  argv_copy = copy_stringv (argv, argc);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_assert (retval == FALSE);
+  g_assert (error != NULL);
+  /* An error occurred, so argv has not been changed */
+  check_identical_stringv (argv_copy, argv);
   g_clear_error (&error);
 
   g_assert (error_func_called);
 
-  g_strfreev (argv);
+  g_strfreev (argv_copy);
+  g_free (argv);
   g_option_context_free (context);
 }
 
