@@ -34,9 +34,9 @@
  * prop3: Defined in TestIface, Implemented in BaseObject, Overridden in DerivedObject
  * prop4: Defined in BaseObject, Overridden in DerivedObject
  */
-   
-static GType base_object_get_type ();
-static GType derived_object_get_type ();
+
+static GType base_object_get_type (void);
+static GType derived_object_get_type (void);
 
 enum {
   BASE_PROP_0,
@@ -154,8 +154,8 @@ base_object_constructor (GType                  type,
   /* The constructor is the one place where a GParamSpecOverride is visible
    * to the outside world, so we do a bunch of checks here
    */
-  GValue value1 = { 0, };
-  GValue value2 = { 0, };
+  GValue value1 = G_VALUE_INIT;
+  GValue value2 = G_VALUE_INIT;
   GParamSpec *pspec;
 
   g_assert (n_construct_properties == 1);
@@ -552,6 +552,7 @@ struct _Base2ObjectClass
   GObjectClass parent_class;
 };
 
+static GType base2_object_get_type (void);
 G_DEFINE_TYPE_WITH_CODE (Base2Object, base2_object, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (TEST_TYPE_IFACE,
                                                 base2_object_test_iface_init))
