@@ -1,3 +1,4 @@
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
 #include <glib-object.h>
 
 static void
@@ -5,7 +6,7 @@ test_param_value (void)
 {
   GParamSpec *p, *p2;
   GParamSpec *pp;
-  GValue value = { 0, };
+  GValue value = G_VALUE_INIT;
 
   g_value_init (&value, G_TYPE_PARAM);
   g_assert (G_VALUE_HOLDS_PARAM (&value));
@@ -24,6 +25,7 @@ test_param_value (void)
   g_param_spec_unref (p2);
 
   g_value_unset (&value);
+  g_param_spec_unref (pp);
 }
 
 static gint destroy_count;
@@ -65,7 +67,7 @@ static void
 test_param_validate (void)
 {
   GParamSpec *p;
-  GValue value = { 0, };
+  GValue value = G_VALUE_INIT;
 
   p = g_param_spec_int ("my-int", "My Int", "Blurb", 0, 20, 10, G_PARAM_READWRITE);
 
@@ -110,8 +112,8 @@ static void
 test_param_convert (void)
 {
   GParamSpec *p;
-  GValue v1 = { 0, };
-  GValue v2 = { 0, };
+  GValue v1 = G_VALUE_INIT;
+  GValue v2 = G_VALUE_INIT;
 
   p = g_param_spec_int ("my-int", "My Int", "Blurb", 0, 20, 10, G_PARAM_READWRITE);
   g_value_init (&v1, G_TYPE_UINT);
@@ -132,8 +134,8 @@ test_param_convert (void)
 static void
 test_value_transform (void)
 {
-  GValue src = { 0, };
-  GValue dest = { 0, };
+  GValue src = G_VALUE_INIT;
+  GValue dest = G_VALUE_INIT;
 
 #define CHECK_INT_CONVERSION(type, getter, value)                       \
   g_assert (g_value_type_transformable (G_TYPE_INT, type));             \
@@ -145,8 +147,15 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_INT_CONVERSION(G_TYPE_CHAR, char, -124)
+  /* Keep a check for an integer in the range of 0-127 so we're
+   * still testing g_value_get_char().  See
+   * https://bugzilla.gnome.org/show_bug.cgi?id=659870
+   * for why it is broken.
+   */
   CHECK_INT_CONVERSION(G_TYPE_CHAR, char, 124)
+
+  CHECK_INT_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_INT_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_INT_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_INT_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_INT_CONVERSION(G_TYPE_INT, int, -12345)
@@ -195,8 +204,8 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_LONG_CONVERSION(G_TYPE_CHAR, char, -124)
-  CHECK_LONG_CONVERSION(G_TYPE_CHAR, char, 124)
+  CHECK_LONG_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_LONG_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_LONG_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_LONG_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_LONG_CONVERSION(G_TYPE_INT, int, -12345)
@@ -220,7 +229,7 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_ULONG_CONVERSION(G_TYPE_CHAR, char, -124)
+  CHECK_ULONG_CONVERSION(G_TYPE_CHAR, char, 124)
   CHECK_ULONG_CONVERSION(G_TYPE_CHAR, char, 124)
   CHECK_ULONG_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_ULONG_CONVERSION(G_TYPE_UCHAR, uchar, 255)
@@ -228,9 +237,9 @@ test_value_transform (void)
   CHECK_ULONG_CONVERSION(G_TYPE_INT, int, 12345)
   CHECK_ULONG_CONVERSION(G_TYPE_UINT, uint, 0)
   CHECK_ULONG_CONVERSION(G_TYPE_UINT, uint, 12345)
-  CHECK_ULONG_CONVERSION(G_TYPE_LONG, long, -12345678)
+  CHECK_ULONG_CONVERSION(G_TYPE_LONG, long, 12345678)
   CHECK_ULONG_CONVERSION(G_TYPE_ULONG, ulong, 12345678)
-  CHECK_ULONG_CONVERSION(G_TYPE_INT64, int64, -12345678)
+  CHECK_ULONG_CONVERSION(G_TYPE_INT64, int64, 12345678)
   CHECK_ULONG_CONVERSION(G_TYPE_UINT64, uint64, 12345678)
   CHECK_ULONG_CONVERSION(G_TYPE_FLOAT, float, 12345678)
   CHECK_ULONG_CONVERSION(G_TYPE_DOUBLE, double, 12345678)
@@ -245,8 +254,8 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_INT64_CONVERSION(G_TYPE_CHAR, char, -124)
-  CHECK_INT64_CONVERSION(G_TYPE_CHAR, char, 124)
+  CHECK_INT64_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_INT64_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_INT64_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_INT64_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_INT64_CONVERSION(G_TYPE_INT, int, -12345)
@@ -270,8 +279,8 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_UINT64_CONVERSION(G_TYPE_CHAR, char, -124)
-  CHECK_UINT64_CONVERSION(G_TYPE_CHAR, char, 124)
+  CHECK_UINT64_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_UINT64_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_UINT64_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_UINT64_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_UINT64_CONVERSION(G_TYPE_INT, int, -12345)
@@ -295,8 +304,8 @@ test_value_transform (void)
   g_value_unset (&src);                                                \
   g_value_unset (&dest);
 
-  CHECK_FLOAT_CONVERSION(G_TYPE_CHAR, char, -124)
-  CHECK_FLOAT_CONVERSION(G_TYPE_CHAR, char, 124)
+  CHECK_FLOAT_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_FLOAT_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_FLOAT_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_FLOAT_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_FLOAT_CONVERSION(G_TYPE_INT, int, -12345)
@@ -320,8 +329,8 @@ test_value_transform (void)
   g_value_unset (&src);                                                 \
   g_value_unset (&dest);
 
-  CHECK_DOUBLE_CONVERSION(G_TYPE_CHAR, char, -124)
-  CHECK_DOUBLE_CONVERSION(G_TYPE_CHAR, char, 124)
+  CHECK_DOUBLE_CONVERSION(G_TYPE_CHAR, schar, -124)
+  CHECK_DOUBLE_CONVERSION(G_TYPE_CHAR, schar, 124)
   CHECK_DOUBLE_CONVERSION(G_TYPE_UCHAR, uchar, 0)
   CHECK_DOUBLE_CONVERSION(G_TYPE_UCHAR, uchar, 255)
   CHECK_DOUBLE_CONVERSION(G_TYPE_INT, int, -12345)
@@ -378,9 +387,9 @@ test_value_transform (void)
   g_value_init (&src, G_TYPE_STRING);
   g_value_init (&dest, G_TYPE_CHAR);
   g_value_set_static_string (&src, "bla");
-  g_value_set_char (&dest, 'c');
+  g_value_set_schar (&dest, 'c');
   g_assert (!g_value_transform (&src, &dest));
-  g_assert_cmpint (g_value_get_char (&dest), ==, 'c');
+  g_assert_cmpint (g_value_get_schar (&dest), ==, 'c');
   g_value_unset (&src);
   g_value_unset (&dest);
 }
